/*
 * SIDB - web based Scientific Image DataBase application.
 *
 * Copyright (C) 2000-2003 Nico Stuurman
 * Copyright (C) 2003-2004 Frank Keijzers
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: database.pgsql,v 1.9 2006/04/26 09:39:32 jose-v Exp $
 */


CREATE SEQUENCE images_imageid_seq
    START 3 INCREMENT 1 MAXVALUE 2147483647 MINVALUE 1 CACHE 1;
SELECT nextval ('"images_imageid_seq"');

CREATE TABLE images
(
    imageid INT4 DEFAULT nextval('images_imageid_seq'::TEXT) NOT NULL,
    filename TEXT NOT NULL,
    description TEXT,
    userid INT4 NOT NULL,
    private BOOL DEFAULT 't'::BOOL,
    restrict_to_group BOOL,
    everyone BOOL,
    trust INT4,
    date_time INT4 NOT NULL,
    exp_date INT4,
    exp_mail_date INT4,
    sensortype CHARACTER VARYING,
    channels INT2,
    numaperture FLOAT8,
    refrinxmedium FLOAT8,
    refrinxlensmedium FLOAT8,
    filetype CHARACTER VARYING,
    filetype_extension CHARACTER VARYING,
    raw_data BOOL,
    microscope_type INT2,
    x_thumb INT2,
    y_thumb INT2,
    fs_thumb INT2,
    x_nm FLOAT8,
    y_nm FLOAT8,
    z_nm FLOAT8,
    t_sec FLOAT8,
    thumb_nr INT2,
    filesize INT8,
    channel_data TEXT,
    mime TEXT,
    settings TEXT,
    x_pixel smallINT,
    y_pixel smallINT,
    z_pixel smallINT,
    compressed BOOL,
    pinsize FLOAT8,
    pinspacing FLOAT8,
    md5sum TEXT
);

CREATE UNIQUE INDEX images_pkey
    ON images USING BTREE (imageid int4_ops);
CREATE INDEX images_userid_idx
    ON images USING BTREE (userid int4_ops);
CREATE INDEX images_private_idx
    ON images USING BTREE (private bool_ops);
CREATE INDEX images_restrict_idx
    ON images USING BTREE (restrict_to_group bool_ops);
CREATE INDEX images_trust_idx
    ON images USING BTREE (trust int4_ops);



CREATE SEQUENCE "users_userid_seq" start 3 increment 1 maxvalue 2147483647 minvalue 1  cache 1 ;
SELECT nextval ('"users_userid_seq"');

CREATE SEQUENCE "groups_groupid_seq" start 4 increment 1 maxvalue 2147483647 minvalue 1  cache 1 ;
SELECT nextval ('"groups_groupid_seq"');

CREATE SEQUENCE "location_locationid_seq" start 1 increment 1 maxvalue 2147483647 minvalue 1  cache 1 ;
CREATE SEQUENCE "template_templateid_seq" start 1 increment 1 maxvalue 2147483647 minvalue 1  cache 1 ;
SELECT nextval ('"template_templateid_seq"');

CREATE SEQUENCE "keywords_keywordid_seq" start 3 increment 1 maxvalue 2147483647 minvalue 1  cache 1 ;
SELECT nextval ('"keywords_keywordid_seq"');

CREATE SEQUENCE "channelinfo_channelinfoid_seq" start 1 increment 1 maxvalue 2147483647 minvalue 1  cache 1 ;

CREATE SEQUENCE "microscopes_microscopeid_seq" start 4 increment 1 maxvalue 2147483647 minvalue 1  cache 1 ;
SELECT nextval ('"microscopes_microscopeid_seq"');

CREATE SEQUENCE "channels_channelid_seq" start 1 increment 1 maxvalue 2147483647 minvalue 1  cache 1 ;
SELECT nextval ('"channels_channelid_seq"');

CREATE SEQUENCE "colors_colorid_seq" start 8 increment 1 maxvalue 2147483647 minvalue 1  cache 1 ;
SELECT nextval ('"colors_colorid_seq"');

CREATE SEQUENCE "projects_projectid_seq" start 1 increment 1 maxvalue 2147483647 minvalue 1 cache 1 ;
SELECT nextval ('"projects_projectid_seq"');


CREATE TABLE "template"
(
	"templateid" INT4 DEFAULT nextval('template_templateid_seq'::TEXT) NOT NULL,
	"userid" INT4,
	"templatename" CHARACTER VARYING,
	"origin" CHARACTER VARYING,
	"scale" CHARACTER VARYING,
	"exp_period" INT4,
	"sensortype" CHARACTER VARYING,
	"channels" INT2,
	"refrinxmedium" FLOAT8,
	"numaperture" FLOAT8,
	"refrinxlensmedium" FLOAT8,
	"description" TEXT,
	"private" BOOL,
	"restrict_to_group" BOOL,
	"raw_data" BOOL,
	"microscope_type" INT2,
	"x_nm" FLOAT8,
	"y_nm" FLOAT8,
	"z_nm" FLOAT8,
	"t_sec" FLOAT8,
	"x_thumb" INT2,
	"y_thumb" INT2,
        "fs_thumb" INT2,
	"everyone" BOOL,
	"trust" INT4,
	"channel_data" TEXT,
	"filetype" CHARACTER VARYING,
	"pinsize" FLOAT8,
	"pinspacing" FLOAT8
);
CREATE TABLE "groups"
(
	"groupid" INT4 DEFAULT nextval('groups_groupid_seq'::TEXT) NOT NULL,
	"groupname" CHARACTER VARYING(20) NOT NULL,
	"adminid" INT4
);
CREATE TABLE "filetypes"
(
	"name" CHARACTER VARYING,
	"extensions" CHARACTER VARYING,
	"sortid" INT2
);
CREATE TABLE "translate"
(
	"translation" CHARACTER VARYING,
	"columnname" CHARACTER VARYING
);
CREATE TABLE "imagexkeywords"
(
	"imageid" INT4 NOT NULL,
	"keywordid" INT4 NOT NULL
);
CREATE TABLE "keywords"
(
	"keywordid" INT4 DEFAULT nextval('keywords_keywordid_seq'::TEXT) NOT NULL,
	"keyword" CHARACTER(50) NOT NULL
);
CREATE TABLE "templatexkeyword"
(
	"templateid" INT4,
	"keywordid" INT4
);
CREATE TABLE "groupxkeywords"
(
	"groupid" INT4 NOT NULL,
	"keywordid" INT4 NOT NULL
);
CREATE TABLE "microscopes"
(
	"microscopeid" INT4 DEFAULT nextval('microscopes_microscopeid_seq'::TEXT) NOT NULL,
	"microscope" TEXT,
    "huygens_scope" TEXT
);
CREATE TABLE "colors"
(
	"colorid" INT4 DEFAULT nextval('colors_colorid_seq'::TEXT) NOT NULL,
	"color_hex" CHARACTER(4),
	"color_text" TEXT,
	"color_sort" INT2,
	"color_string" CHARACTER(11)
);
CREATE TABLE "auditors"
(
	"userid" INT4,
	"groupid" INT4
);
CREATE TABLE "users"
(
	"userid" INT4 DEFAULT nextval('users_userid_seq'::TEXT) NOT NULL,
	"login" TEXT NOT NULL,
	"pwd" TEXT NOT NULL,
	"groupid" INT4,
	"real_name" TEXT,
	"status" INT4 DEFAULT 0,
	"active" BOOL,
	"email" TEXT,
	"homedir" TEXT,
	"settings" TEXT,
	"last_image" INT4,
	"last_action" INT4,
	"outdir" TEXT,
	"login_defunct" TEXT
);
CREATE TABLE "mime"
(
	"mime" TEXT,
	"extension" TEXT
);
CREATE TABLE "projects"
(
	"projectid" INT4 DEFAULT nextval('projects_projectid_seq'::TEXT) NOT NULL,
	"name" TEXT,
	"description" TEXT,
	"userid" INT4,
	"rights" INT2
);
CREATE TABLE "projectximages"
(
	"projectid" INT4 NOT NULL,
	"imageid" INT4 NOT NULL
);
CREATE TABLE "projectxprojects"
(
	"motherid" INT4 NOT NULL,
	"daughterid" INT4 NOT NULL
);
CREATE TABLE "projectxtemplate"
(
	"projectid" INT4 NOT NULL,
	"templateid" INT4 NOT NULL
);
CREATE TABLE "imagextrust"
(
	"imageid" INT4 NOT NULL,
	"userid" INT4 NOT NULL
);





COPY "groups" FROM stdin;
4	guests	\N
\.
COPY "images" FROM stdin;
3	Janis.gif	\N	1	f	f	t	0	977845235	\N	\N	\N	\N	\N	\N	\N	2-D	GIF	t	\N	256	256	\N	\N	\N	\N	\N	8	17501	\N	image/gif	\N	\N	\N	\N	f	\N	\N	\N
\.
COPY "template" FROM stdin;
\.
COPY "filetypes" FROM stdin;
2-D	tiff;tif;jpg;jpeg;png;gif;psd	1
multipage	\N	10
3-D	TIF;tif;pic;PIC	20
ICS	ICS;IDS;ics;ids	21
other	\N	30
\.
COPY "translate" FROM stdin;
Position of the origin	origin
Samplingdistance	scale
Number of channels	channels
Refractionindex of the medium	refrinxmedium
Name of the image	imagename
Description	description
Private or public	sharing
Time and date	date_time
Template name	templatename
Excitation wavelength	lambdaex
Emission wavelength	lambdaem
My group:	restrict_to_group
Refraction index of immersion medium	refrinxlensmedium
File name	filename
Me:	private
2-D, 3-D, or ICS	filetype
File type	filetype_extension
Raw data	raw_data
File size	filesize
Pinhole radius	pinholeradius
Microscope type	sensortype
Excitation photons	exphotoncnt
Numerical aperture of the lens	numaperture
Everyone:	everyone
\.
COPY "imagexkeywords" FROM stdin;
3	2
3	3
\.
COPY "keywords" FROM stdin;
2	Janis                                             
3	baby                                              
\.
COPY "templatexkeyword" FROM stdin;
\.
COPY "groupxkeywords" FROM stdin;
0	2
0	3
\.
COPY "microscopes" FROM stdin;
1	Confocal	confocal
2	Wide field	widefield
3	4Pi excitation	4pi
4	Generic sensor	generic
5	Nipkow disk	nipkow
\.

COPY "colors" FROM stdin;
1	#0f0	green	1000	0 255 0    
2	#f00	red	2000	255 0 0    
3	#00f	blue	3000	0 0 255    
4	#fff	b/w	4000	255 255 255
5	#000	black	5000	0 0 0      
6	#bbb	grey 75%	4200	192 192 192
7	#888	grey 50%	4400	128 128 128
8	#444	grey 25%	4600	64 64 64   
9	#0ff	cyan	3200	0 255 255
10	#f0f	magenta	3400	255 0 255
11	#ff0	yellow	3600	255 255 0
\.
COPY "auditors" FROM stdin;
\.
COPY "users" FROM stdin;
1	sysadmin	psswd	0	system administrator	300	t			a:4:{s:8:"filetype";s:3:"2-D";s:10:"templateid";s:1:"0";s:10:"uploadtype";s:4:"html";s:10:"nr_visible";s:1:"5";}	4	977845452		\N
2	guest	guest	4	Example user of sidb	20	t			a:1:{s:10:"nr_visible";i:10;}	\N	\N		defunct
\.
COPY "mime" FROM stdin;
image/jpeg	JPG
image/jpeg	JPEG
video/quicktime	MOV
image/gif	GIF
image/tiff	TIFF
image/tiff	TIF
application/ppt	PPT
application/pdf	PDF
image/x-bmp	BMP
\.
CREATE UNIQUE INDEX "groups_pkey" on "groups" USING BTREE ( "groupid" "int4_ops" );
CREATE UNIQUE INDEX "groups_groupname_key" on "groups" USING BTREE ( "groupname" "varchar_ops" );
CREATE UNIQUE INDEX "template_pkey" on "template" USING BTREE ( "templateid" "int4_ops" );
CREATE UNIQUE INDEX "filetypes_name_key" on "filetypes" USING BTREE ( "name" "varchar_ops" );
CREATE UNIQUE INDEX "translate_columnname_key" on "translate" USING BTREE ( "columnname" "varchar_ops" );
CREATE UNIQUE INDEX "keywords_keywordid_key" on "keywords" USING BTREE ( "keywordid" "int4_ops" );
CREATE UNIQUE INDEX "microscopes_microscopeid_key" on "microscopes" USING BTREE ( "microscopeid" "int4_ops" );
CREATE UNIQUE INDEX "colors_colorid_key" on "colors" USING BTREE ( "colorid" "int4_ops" );
CREATE UNIQUE INDEX "users_pkey" on "users" USING BTREE ( "userid" "int4_ops" );
CREATE  INDEX "users_groupid_idx" on "users" USING BTREE ("groupid" "int4_ops");
CREATE  INDEX "users_login_idx" on "users" USING BTREE ("login" "varchar_ops");
CREATE  INDEX "groupxkeywords_groupid_idx" on "groupxkeywords" USING BTREE ( "groupid" "int4_ops" );
CREATE  INDEX "groupxkeywords_keywordid_idx" on "groupxkeywords" USING BTREE ( "keywordid" "int4_ops" );
CREATE  INDEX "imagexkeywords_imageid_idx" on "imagexkeywords" USING BTREE ( "imageid" "int4_ops" );
CREATE  INDEX "imagexkeywords_keywordid_idx" on "imagexkeywords" USING BTREE ( "keywordid" "int4_ops" );
