<?php

/*
 * This file contains the class Image.
 */


class Image
{

    // global vars
    var $imageid, $filename, $description;
    var $channelcnt, $mtype, $na, $rilens, $rimedium, $samplesizes;
    var $addid, $modid, $firsttime, $lasttime;
    var $lambdaex, $lambdaem, $photoncnt, $pinhole, $pinholedist;

    // static member function to get an existing Image by its imageid.
    function getImage($imageid)
    {
    global $S_USERID;
	// make sure $imageid is really a valid integer.
	if (!isset($imageid) || ereg("[^0-9]", $imageid))
	{
	    error_log("Invalid imageid $imageid for getImage.");
	    return NULL;
	}

    if (Image::viewrights($imageid,$S_USERID) == false) {
        error_log("You are not allowed to use image $imageid.");
        return NULL;
    }


	$database = new Database("SELECT * FROM images ".
	                         "WHERE imageid='$imageid'");
	if ($database->queryRows() != 1)
	{
	    error_log("Invalid database image $imageid for getImage.");
	    return NULL;
	}
	else
	{
            $db_array = $database->queryArray(0);
	    return new Image($db_array);
	}
    }


    ////
    // !function to determine whether an user has the right to see an image
    function viewrights ($im_id, $user) {
       $query  = imgright_query ($user, "filename", "(imageid = $im_id)").";";
       $search_result = new Database($query);
       if ($search_result->queryRows() != 1)
       {
           error_log("You are not allowed to see this image");
           return false;
       }
       else
       {
           return true;
       }
    }

    // static member function to set a new Image by its owner.
    function setImage($userid)
    {
	// make sure $userid is really a valid integer.
	if (!isset($userid) || ereg("[^0-9]", $userid))
	{
	    error_log("Invalid userid $userid for setImage.\n");
	    return NULL;
	}

	// create empty information array for new user
	$db_array = array();
	$db_array["userid"] = $userid;
	$db_array["modid"] = $userid;
	$db_array["firsttime"] = time();
	$db_array["lasttime"] = time();

	// create Image object and return it.
	return new Image($db_array);
    }



    /*
     * Constructor for creating new Image object with parameters in $db_array.
     * Don't call this directly, but use getImage or setImage.
     */
    function Image($db_array)
    {
	if (!is_array($db_array))
	{
	    print("Invalid array for Image.");
	}
	$this->setImageId($db_array["imageid"]);
	$this->setFileName($db_array["filename"]);
	$this->setDescription($db_array["description"]);

	$this->setChannelCnt($db_array["channels"]);

	$this->setMType($db_array["microscope_type"]);
	$this->setNA($db_array["numaperture"]);
	$this->setRILens($db_array["refrinxlensmedium"]);
	$this->setRIMedium($db_array["refrinxmedium"]);
	$this->setSampleSizes($db_array["x_nm"]. " ". $db_array["y_nm"] ." ". $db_array["z_nm"]);
        #$this->setLambdaEm($db_array["l

	$this->setAddId($db_array["userid"]);
	$this->setModId($db_array["modid"]);
	$this->setFirstTime($db_array["firsttime"]);
	$this->setLastTime($db_array["lasttime"]);

        $channeldata = unserialize ($db_array["channel_data"]) ;
        $lex = array();
        $lem = array();
        $phcnt = array();
        $pinh = array();

        $this->setPinholeDist($db_array["pinspacing"]);

        for ($i = 0; $i < $this->channelcnt ; $i++ ) {
              $lex[$i] = $channeldata[$i]["ex"];
              $lem[$i] = $channeldata[$i]["em"];
              $phcnt[$i] = $channeldata[$i]["exphtcnt"];
              $pinh[$i] = $channeldata[$i]["pinholeradius"];
        }
        $this->setLambdaEx($lex);
        $this->setLambdaEm($lem);
        $this->setPhotonCnt($phcnt);
        $this->setPinhole($pinh);
    }

    // functions to query Image variables.
    function getImageId() { return $this->imageid; }
    function getFileName() { return $this->filename; }
    function getArchiveName() { return $this->imageid."_".$this->filename; }
    function getDescription() { return $this->description; }

    function getChannelCnt() { return $this->channelcnt;}
    function getMType() { return $this->mtype;}
    function getMTypeTxt() {

        $database = new Database("SELECT huygens_scope FROM microscopes ".
                                 "WHERE microscopeid='".$this->mtype."'");
        if ($database->queryRows() != 1)
        {
            error_log("Invalid database microscope type ".$this->mtype.".");
            return NULL;
        }
        else
        {
            $db_array = $database->queryArray(0);
            return $db_array['huygens_scope'];
        }
    }

    function getNA() { return $this->na;}
    function getRILens() { return $this->rilens;}
    function getRIMedium() { return $this->rimedium;}
    function getSampleSizes() { return $this->samplesizes; }
    function getLambdaEm() { return $this->lambdaem; }
    function getLambdaEx() { return $this->lambdaex; }
    function getPhotonCnt() { return $this->photoncnt; }
    function getPinhole() { return $this->pinhole; }
    function getPinholeDist() { return $this->pinholedist; }

    function getAddId() { return $this->addid; }
    function getModId() { return $this->modid; }
    function getFirstTime() { return $this->firsttime; }
    function getLastTime() { return $this->lasttime; }

    // functions to set Image variables.
    function setImageId($imageid) { $this->imageid = $imageid; }
    function setFileName($filename) { $this->filename = $filename; }
    function setDescription($description) { $this->description = $description; }

    function setChannelCnt($channelcnt) { $this->channelcnt = $channelcnt;}
    function setMType($mtype) { $this->mtype = $mtype;}
    function setNA($na) { $this->na = $na;}
    function setRILens($rilens) { $this->rilens = $rilens;}
    function setRIMedium($rimedium) { $this->rimedium = $rimedium;}
    function setSampleSizes($samplesizes) { $this->samplesizes = $samplesizes; }
    function setLambdaEm($lambdaem) { $this->lambdaem = $lambdaem; }
    function setLambdaEx($lambdaex) { $this->lambdaex = $lambdaex; }
    function setPhotonCnt($photoncnt) { $this->photoncnt = $photoncnt; }
    function setPinhole($pinhole) { $this->pinhole = $pinhole; }
    function setPinholeDist($pinholedist) { $this->pinholedist = $pinholedist; }


    function setAddId($addid) { $this->addid = $addid; }
    function setModId($modid) { $this->modid = $modid; }
    function setFirstTime($firsttime) { $this->firsttime = $firsttime; }
    function setLastTime($lasttime) { $this->lasttime = $lasttime; }

    // GENERAL GET/SET FUNCTIONS - TESTING
    // ok this construction works;-)
    function getAttribute($attribute)
    {
	return $this->$attribute;
    }
	



    
    /*
     * Database section
     */

    function databaseUpdate()
    {
	// Set query to update the image in database
	$query =
	    "UPDATE images SET ".
	    "filename='".pg_escape_string($this->filename) ."', ".
	    "modid='$this->modid', ".
	    "firsttime='$this->firsttime', ".
	    "lasttime='$this->lasttime' ".
	    "WHERE imageid='$this->imageid'";

	$database = new Database($query);

	if ($database->queryRowsAffected() != 1)
	{
	    return FALSE;
	}
	return TRUE;
    }

    function databaseInsert()
    {
	// Set query to insert the new image in database.
	$query  =
	    "INSERT INTO images (".
	    "filename, ".
	    "userid, ".
	    "modid, ".
	    "firsttime, ".
	    "lasttime) ".
	    "VALUES (".
	    "'".pg_escape_string($this->filename) ."', ".
	    "'$this->addid', ".
	    "'$this->modid', ".
	    "'$this->firsttime', ".
	    "'$this->lasttime')";

	$database = new Database($query);

	if ($database->queryRowsAffected() != 1)
	{
	    return FALSE;
	}

	// Get image from database to report fail or
	// set its imageid and report success.
	$query =
	    "SELECT * FROM images WHERE ".
	    "filename='$this->filename' AND ".
	    "userid='$this->addid' AND ".
	    "modid='$this->modid' AND ".
	    "firsttime='$this->firsttime' AND ".
	    "lasttime='$this->lasttime'";

	$database->setQuery($query);
	
	if ($database->queryRows() != 1)
	{
	    return FALSE;
	}

	$array = $database->queryArray(0);
	$this->setImageid($array["imageid"]);
	return TRUE;
    }

}

?>
