<?php

  /**************************************************************************\
  * export_inc.php    Takes care of authorization.                          *
  ***************************************************************************
  * Contains functions facilitating export of data                           *
  * function save_as_ics($location_name, $imageid                            *
  *                                                                          *
  * Authors: SVI group University Utrecht and Nico Stuurman                  *
  * email: stuurman@rulbim.leidenuniv.nl                                     *
  * demo at: https://impi.leidenuniv.nl                                      *
  * Created: 21/02/01                                                        *
  * Last modified 22/02/01                                                   *
  * Copyright (c) 2000 by Nico Stuurman                                      *
  * ------------------------------------------------------------------------ *
  *  This program is free software; you can redistribute it and/or modify it *
  *  under the terms of the GNU General Public License as published by the   *
  *  Free Software Foundation; either version 2 of the License, or (at your  *
  *  option) any later version.                                              *
  \**************************************************************************/


// function to save a file in ics format with correct settings
function save_as_ics($imageid) {
   global $S_USERID, $S_OUTDIR, $include_dir, $archive_dir, $thumbnail_dir, $image_tool;
    require ("$include_dir/show_base_inc.php");
   if (!imgviewrights ($imageid, $S_USERID)) {
      echo "<h3>Insufficient rights for the requested operation </h3>";
      printfooter ();
      exit ();
   }

   // read in refr. indexes, wavelengths, photons, etc...
   $query = "SELECT filename, refrinxmedium, refrinxlensmedium, numaperture, microscope_type, x_nm, y_nm, z_nm, t_sec, pinsize, pinspacing  FROM images WHERE imageid='$imageid';";
   $result = db_exec ($query);
   $r = db_fetch_array($result,0);
   if (!$r) {
      echo "<h3>Serious error reading data</h3>";
      exit ();
   }

   while (list($key, $value) = each ($r)) {
      ${$key} = $value;
   }

   if (! function_exists (get_channels))
      include ("$include_dir/channel_inc.php");

   $channels = get_channels ($imageid, 1, $nr_channels);

   if ($microscope_type) {
      $resultm=db_exec("SELECT huygens_scope FROM microscopes WHERE microscopeid='$microscope_type'");
      $rm=db_fetch_array($resultm,0);
      $microscope=$rm["huygens_scope"];
   }   

   // strip the suffix from the filename
   $newname = substr ($filename, 0, strrpos($filename, "."));
 
   // create unique filename for huscript 
   // (Huygens Scripting Software http://www.svi.nl)
   $huygens_script = $archive_dir ."/tmp/" . $S_USERID . time(). ".script";

   // create the script for huscript
   $fo = fopen($huygens_script, "w");
   $path_file = get_image_location ($imageid, false, $location_name, $S_USERID);


   $ics_script .= 'set myFullPath '.$path_file ."\n";
   $ics_script .= 'set myImage [img open $myFullPath]'."\n"; 
   if ($numaperture)
      $ics_script .= '$myImage setp -a ' . "$numaperture \n";
   if ($refrinxlensmedium)
      $ics_script .= '$myImage setp -ril ' . "$refrinxlensmedium \n";
   if ($refrinxmedium)
      $ics_script .= '$myImage setp -ri ' . "$refrinxmedium \n";
   if ($microscope_type)
      $ics_script .= '$myImage setp -micr ' . "$microscope \n";
   if ($pinsize)
      $ics_script .= '$myImage setp -pr ' . "$pinsize \n";
   if ($pinspacing)
      $ics_script .= '$myImage setp -ps ' . "$pinspacing \n";
   if ($x_nm && $y_nm && $z_nm) {
      $x_nm=$x_nm/1000;
      $y_nm=$y_nm/1000;
      $z_nm=$z_nm/1000;
      $ics_script .= '$myImage setp -s {' . "$x_nm $y_nm $z_nm $t_sec".'}'." \n";
   }
   // since this sets the first channel over and over again, and I don't knwo how to do the others, stop after 1 channel
   if ($nr_channels) $nr_channels = 1;
   for ($i = 0; $i < $nr_channels; $i++) {
      if ($a = $channels[$i]["ex"])
         $ics_script .= '$myImage setp -ex ' . "$a \n";
      if ($a = $channels[$i]["em"])
         $ics_script .= '$myImage setp -em ' . "$a \n";
      if ($a = $channels[$i]["pinholeradius"])
         $ics_script .= '$myImage setp -pr ' . "$a \n";

   } 
   
   $ics_script .= '$myImage save ' . "$S_OUTDIR/$newname -type ics \n";
   $ics_script .= '$myImage ' . "del \n";
   $ics_script .= "exit \n";

   fwrite ($fo , $ics_script);
   fclose($fo);

   // execute the huygens program to convert the ics file to a tiff
   $exec_string = "$image_tool -script $huygens_script -noTk";
   Debug::printDebug("exec_string: $exec_string");
   exec ($exec_string); 


   // remove script file 
   $huygensresult = exec ("rm -f $huygens_script");

   $filename = "$S_OUTDIR/$newname.ics";
   $temp_dir = basename ($S_OUTDIR);
   $meta = "<html><body>\n";
   $meta .= display_meta_data(false);
   $meta .= "\n</body></html>";
   $metafile = str_replace(".ics",".html",$filename);
   $fo = fopen($metafile, "w");
   fwrite ( $fo, $meta);
   fclose($fo);

   if (file_exists ($filename)) { 
      $newname = basename ($filename);
      echo "<h3>File <i>$newname</i> is saved together with its <i>ids</i> counterpart in your directory <i>$temp_dir</i></h3>\n";  
   }
   else
      echo "<h3>Tried to save file <i>$newname</i> in your directory <i>$temp_dir</i>, but something went wrong. Sorry.</h3>\n";

}

?>
