<?php

//install.php - Database Initialization and maintenance.
//install.php - author: Nico Stuurman (<nicos@itsa.ucsf.edu>)
  /***************************************************************************
  * When database is not installed, this script will install the postgresql  *
  *   tables. The files config_inc, index.php, and include.php will be re-   *
  *   written to reflect the local set-up                                    *
  ***************************************************************************
  * function error_not_owned_error ($webdaemon, $localdir, $script)          *
  * function checkd ($daemon)                                                *
  * function which ($app)                                                    *
  * function movedir ($dirnew, $dirold, $type, $webdaemon)                   *
  * function changeline (&$array, $string, $value)                           *
  * function config_form ($first_time)                                       *
  * function uncompress_all()                                                *
  * function process_config_form ()                                          *
  * function move_config_file ($localdir, $the_file, $webdaemon)             *
  ***************************************************************************
  * Authors: SVI group University Utrecht and Nico Stuurman                  *
  * email: stuurman@rulbim.leidenuniv.nl                                     *
  * demo at: https://impi.leidenuniv.nl                                      *
  * home: http://sidb.sourceforge.net                                        *
  * Copyright (c) 2000, 2001, 2002 by Nico Stuurman                          *
  * ------------------------------------------------------------------------ *
  *  This program is free software; you can redistribute it and/or modify it *
  *  under the terms of the GNU General Public License as published by the   *
  *  Free Software Foundation; either version 2 of the License, or (at your  *
  *  option) any later version.                                              *
  \**************************************************************************/




// register variables
$PHP_SELF = $HTTP_SERVER_VARS["PHP_SELF"];
$action = $HTTP_POST_VARS["action"];
$newdb_name = $HTTP_POST_VARS["newdb_name"];
$newdb_user = $HTTP_POST_VARS["newdb_user"];
$newdb_pass = $HTTP_POST_VARS["newdb_pass"];
$newsys_pass = $HTTP_POST_VARS["newsys_pass"];
$postgres_path = $HTTP_POST_VARS["postgres_path"];

// set code version number (as opposed to version in config_inc.php
$version_code = "0.41";

////
// Issues an error messages when essential files can not be written
// by the daemon running the webserver
function error_not_owned_error ($webdaemon, $localdir, $script) {
   if (is_dir ($script))
      $first_word = "Directory";
   if (is_file ($script ))
      $first_word = "File";
   echo "<h3>$first_word <i>$script</i> (and possibly others) are not 
         owned by the user <b>$webdaemon</b>.<br>";
   echo "As root issue the command: 
         <br><br>chown -R $webdaemon $localdir<br><br>";
   echo "Then reload this page.</h3>";
   printfooter ();
}

////
// checks whether a daemon is running.  
// For Mac OS X this only works for postmaster/postgres
function checkd($daemon) 
{  
   if ($daemon=="postmaster")
      $owner = "postgres";
   $daemon = substr ($daemon, 0, 7); 
   // deal with bsd, OS X top
   $server = php_uname();
   if (strstr ($server,"Darwin")) 
      $ps = "/bin/ps -ax | /usr/bin/grep $daemon | /usr/bin/wc -l";
   else
      $ps ="ps -e | grep $daemon | wc -l";  
   $result = exec($ps);  
   if ($result < 1)   
      return false;
   else
      return true;
}

////
// checks for the presence of $app on the local system
function which ($app) {
   $result = exec ("which $app");
   // look in some places

   // correct output for linux and bsd
   if (strstr ($result, "no $app")) 
      $result = false;
   elseif (strstr($result, "$app not found"));
      $result = false; 

   if (!$result) 
      if (file_exists("/usr/local/bin/$app"))
         $result = "/usr/local/bin/$app";
   if (!$result) 
      if (file_exists("/usr/local/sbin/$app"))
         $result = "/usr/local/sbin/$app";
   if (!$result) 
     if (file_exists("/usr/bin/$app"))
         $result = "/usr/bin/$app";
   if (!$result) 
      if (file_exists("/usr/sbin/$app"))
         $result = "/usr/sbin/$app";
   if (!$result) 
      if (file_exists("/usr/local/svi/bin/$app"))
         $result = "/usr/local/svi/bin/$app";
   if (!$result) 
      if (file_exists("/usr/X11R6/bin/$app"))
         $result = "/usr/X11R6/bin/$app";
   if (!$result) 
      if (file_exists("/usr/X11/bin/$app"))
         $result = "/usr/X11/bin/$app";
   
   return $result;
}

////
// Creates dirnew if it does not exist and moves the content of dirold to dirnew
function movedir ($dirnew, $dirold, $type, $webdaemon) {
   if ($type<>"archive" && $type<>"thumbnail")
      return false;
   if ($dirnew <> $dirold) { 
      // if it does not exit yet, create dirnew
      if (! file_exists($dirnew) )
         exec ("mkdir -p -m 0700 $dirnew");
      if (is_writeable ($dirnew)) {
         exec ("mv $dirold/* $dirnew");
            if (is_readable ($dirnew)) { 
               echo "<h3>Moved $type to $dirnew.</h3>\n";
               return true;
            }
            else
                echo "<h3>Failed to move the $type directory</h3>";
      }
      else {
         echo "<h3>User <b>$webdaemon</b> can not write to directory 
                \"$dirnew\".\n";
         echo "The $type has not been moved.</h3>\n";
      }
   } 
   return false;
}

////
// changes lines in an array (read from config_file here).  
// When a variable is not found, it is inserted right before the closing tag. 
function changeline (&$array, $string, $value) {
   if ( (gettype ($value)=="string") && ($value<>"true") && ($value <> false) )
      $value = "\"$value\"";
   if (!$value) 
      $value = "false";
   // echo "$string: $value<br>";
   while ($i < sizeof ($array) && !$test) {
      if (strstr ($array[$i], '$'.$string)) {
         if (!(strstr ($array[$i], "//") )) {
            if (!(strstr ($array[$i],$string .")"))) {
               $test = true;
               $found = true;
               $array[$i] = '$'."$string = $value;\n";
            }
         }
      }
      // remember where we want to insert
      if (strstr ($array[$i], "DO NOT EDIT BELOW"))
         $lastline = $i;
      if (strstr($array[$i], "$title = "))
         $lastline = $i;
      $i++;
   }
   if (!$found) {
      // this is a new variable; 
      // append just before DO NOT EDIT LINE in config_inc, before $title in 
      //   index.php
      if ($lastline) {
         $temparray[] = '$'."$string = $value;\n";
         $temparray2 = array_slice ($array, $lastline);
         $temparray3 = array_merge ($temparray, $temparray2);
         array_splice ($array, $lastline);
         $array = array_merge ($array, $temparray3);
      }
      // this should not happen, but better try something
      else {
         $end = $array[$i-1];
         $array[$i-1] = '$'."$string = $value;\n";
         $array[$i] = $end;
      }
   }
} 

////
// Display the form with configuration settings
function config_form ($first_time) {
   global $archive_dir, $thumbnail_dir, $include_dir, $IMAGE_TOOLS;
   global $localdir, $image_tool, $convert_path, $movie_tool;
   global $use_imagej, $max_size_for_convert, $max_size_for_movie;
   global $max_thumb_size, $default_thumb_size, $min_thumb_size;
   global $people_dir, $impose_folder, $infolder, $outfolder;
   global $max_folder_size, $max_in_gallery, $PHP_SELF, $newsys_pass;
   global $configarray, $version, $make_rotating_movie, $date_format, $SQL_log;
   global $use_sessions, $secure_server, $safe_login, $pam_prg;
   global $bzip2,$compress_delay; 
   global $auth_method,$nis_domain,$user_autocreate,$auto_group,$max_md5_filesize;
   global $TD_only,$logo,$logo_x_size,$logo_y_size,$logo_link;
   global $emailNotifyList, $emailNotifyAddress, $emailSenderAddress;

   // initialize with sensible values, and write these to config_inc
   if ($first_time) {
      $archive_dir = "$localdir/archive";
      changeline ($configarray, "archive_dir", $archive_dir);
      $thumbnail_dir = "$localdir/thumbnails";
      changeline ($configarray, "thumbnail_dir", $thumbnail_dir);
      $include_dir = "$localdir/includes"; 
      changeline ($configarray, "include_dir", $include_dir);
      $IMAGE_TOOLS = false;
      $image_tool = which ("huscript");
      // The 'huygens2t' executable has been renamed to 'huscript'
      // Keep next line for old Huygens Suites
      if(!$image_tool) $image_tool = which ("huygens2t");
      if($image_tool) $IMAGE_TOOLS = true;
      changeline ($configarray, "IMAGE_TOOLS", $IMAGE_TOOLS);      
      changeline ($configarray, "image_tool", $image_tool);
      $convert_path = which ("convert");
      changeline ($configarray, "convert_path", $convert_path);
      if (! ($movie_tool = which ("ffmpeg")))
         if (! ($movie_tool = which ("mpeg2encode")) )
            $movie_tool = which ("convert");
      changeline ($configarray, "movie_tool", $movie_tool);
      $memory_limit = get_cfg_var ("memory_limit");
      if (strstr ($memory_limit, "M"))
         $memory_limit = 1048576 * $memory_limit;
      if ($memory_limit) {
         $max_size_for_convert = floor ($memory_limit/5);
         $max_size_for_movie = floor ($memory_limit/10);
      }
      else {
         $max_size_for_convert = 33554432;
         $max_size_for_movie =   12000000;
      }
      if (file_exists ("$localdir/ij.jar"))
         $use_imagej = true;
      changeline ($configarray, "use_imagej", $use_imagej);
      $max_thumb_size = 512;
      $default_thumb_size = 300;
      $min_thumb_size = 100;
      if (file_exists ("/home")) 
         $people_dir = "/home";
      elseif (file_exists ("/usr/people"))
         $people_dir = "/usr/people";
      $impose_folder = true;
      $infolder = "tosidb";
      $outfolder = "fromsidb";
      $max_folder_size = 1073741824;
      $max_in_gallery = 40; 
      $max_md5_filesize = 8000000;
      $make_rotating_movie = true;
      $testpwd = which ("testpwd");
      $pam_prg = "sudo " . $testpwd;
      $bzip2 = which("gzip");
      $month=mktime(0,0,0,2,1,2000)-mktime(0,0,0,1,1,2000);
      $compress_delay = $month;
      $logo="images/svilogo.gif";
      $logo_x_size=95;
      $logo_y_size=50;
      $logo_link="http://sidb.sourceforge.net";

      // write changed values back to file
      if (!(@chmod ("$localdir/includes/config_inc.php", 0700))) {
         error_not_owned_error ($webdaemon, "$localdir/includes", 
                                "config_inc.php");
         exit ();
      }
      $fconfig = fopen ("$localdir/includes/config_inc.php", "w");
      for ($i = 0; $i<sizeof($configarray); $i++) 
         fwrite ($fconfig, $configarray[$i]);
      fclose ($fconfig); 
   }

   if ($version < "0.14")
      $version = "prior to 0.14";
   echo "<h3 align=center>Scientific Image Database (sidb) 
          version $version.</h3>\n";

   if ($version < "0.16004") {
      $mpeg2encode = which ("mpeg2encode");
      if ($mpeg2encode)
         $movie_tool = $mpeg2encode;
   }

   if ($version < 0.1807) {
      $ffmpeg = which ("ffmpeg");
      if ($ffmpeg)
        $movie_tool = $ffmpeg;
   }

   echo "<form enctype='multipart/form-data' method='post' action=$PHP_SELF>\n";    

   // 'vacuum' button
   if (!$first_time) {
      echo "<table border=1 align=center>\n";
      echo "<tr><td colspan=2><h3 align=center>Database maintenance</h3></td>
            </tr>\n";
      echo "<tr><td>Sweep, vacuum, clean and broom?<br>
             Do this about once a week, or more often when the system's 
             performance decreases.<br>
             This action will also remove images whose expiration date 
             expired.</td>\n";    
      echo "<td align='center'><input align=center type=submit 
             name=action value=Vacuum></td></tr>\n";  
      echo "</table><br>\n";
   }

   // table-form with settings
   echo "<br><br>\n";
   echo "<h3 align=center>Adjust the system-wide settings</h3>\n";  

   echo "<table border=1>\n";    

   echo "<tr><td colspan=2 align=center><h4>Archive and thumbnail directories</h4></td></tr>\n"; 
   echo "<tr><td>Directory for <b>image archive</b>.  
         <br>This directory should not be (directly) accessible through the 
         web.<br> 
         You may enter a location here that your webdaemon can write to. <br>
         SIDB will try to move the archive to the new location.</td>\n";
   $archive_dir_length = strlen ($archive_dir) +10;
   echo "<td><input type='text' name='archive_dir_new' value='$archive_dir' 
         size='$archive_dir_length'></td></tr>\n";

   echo "<tr><td>Directory for <b>thumbnails</b>.  <br>
         This directory should not be (directly) accessible through the web.<br>
         You may enter a location here that your webdaemon can write to.<br>
         SIDB will try to move the thumbnails to the new location.</td>\n";
   $thumbnail_dir_length = strlen ($thumbnail_dir) + 10;
   echo "<td><input type='text' name='thumbnail_dir_new' value='$thumbnail_dir' 
         size='$thumbnail_dir_length'></td></tr>\n";

   echo "<tr><td colspan=2 align=center><h4>Helper applications</h4></td></tr>\n";

   echo "<tr><td>Location of Imagemagick's convert.</td>\n";
   $convert_path_length = strlen ($convert_path) + 10;
   echo "<td><input type='text' name='convert_path_new' value='$convert_path' 
         size='$convert_path_length'></td></tr>\n";

   echo "<tr><td>Maximum file-size to be handled by convert.<br>  Do not exceed 
         50% of the available RAM on your system.</td>";
   echo "<td><input type='text' align='right' name='max_size_for_convert_new' 
         value='$max_size_for_convert'>bytes</td></tr>\n";

   echo "<tr><td>Are you using <a href=http://www.svi.nl/products/software/scripting/>huscript</a>? <br>You only need this application 
         when you want to upload 3-D images from confocal microscopes.</td>\n
         <td>";
   if ($IMAGE_TOOLS)
      echo "Yes <input type='radio' name='IMAGE_TOOLS_new' checked value='Yes'>
            &nbsp&nbsp No<input type='radio' name='IMAGE_TOOLS_new' value='No'>
            ";
   else 
      echo "Yes <input type='radio' name='IMAGE_TOOLS_new' value='Yes'>
            &nbsp&nbsp No<input type='radio' name='IMAGE_TOOLS_new' checked 
            value='No'>\n";
   echo "</td></tr>\n";

   echo "<tr><td>Location of huscript.<br> (you only need this if you answered
         yes to the question above)</td>\n";
   $image_tool_length = strlen ($image_tool)+10;
   echo "<td><input type='text' name='image_tool_new' value='$image_tool' 
         size='$image_tool_length'></td></tr>\n";
 
   echo "<tr><td>Do you want your users to use ImageJ (send as an applet).  
         You need (the patched version of) ij.jar in your sidb directory for 
         this to work</td>\n<td>";
   if ($use_imagej)
      echo "Yes <input type='radio' name='use_imagej_new' checked value='Yes'>
            &nbsp&nbsp No<input type='radio' name='use_imagej_new' value='No'>
            \n";
   else 
      echo "Yes <input type='radio' name='use_imagej_new' value='Yes'>
            &nbsp&nbsp No<input type='radio' name='use_imagej_new' checked 
            value='No'>\n";
   echo "</td></tr>\n";
   
   echo "<tr><td>Location of the movie tool (only needed for 3-D, ICS and multipage 
         images).<br>  Use mpeg2enc from the <a href='http://mjpeg.sourceforge.net'>mjpegtools</a> (fast and good, recomended), <a href='http://ffmpeg.sourceforge.net'>ffmpeg</a> (fast but ugly), mpeg2encode (slow) or 
         Imagemagick's convert (needs mpeg2encode, slow).
         </td>\n";
   $movie_tool_length = strlen ($movie_tool) + 10;
   echo "<td><input type='text' name='movie_tool_new' value='$movie_tool' 
         size='$movie_tool_length'></td></tr>\n";

   echo "<tr><td>Maximum file-size to be handled by the movie tool.<br>  
         Do not exceed 10% of the available RAM on your system.</td>";
   echo "<td><input type='text' align='right' name='max_size_for_movie_new' 
         value='$max_size_for_movie'>bytes</td></tr>\n";

   echo "<tr><td colspan=2 align=center><h4>Login options</h4></td></tr>\n";
   echo "<tr><td>Do you want to authenticate users through sessions ";
   echo "(recomended, but it uses cookies), or with a http popup screen?";
   echo "</td>\n<td>";
   if ($use_sessions)
      echo "Session <input type='radio' name='use_sessions_new' checked value='Yes'>
            &nbsp&nbsp Http<input type='radio' name='use_sessions_new' value='No'>
            \n";
   else 
      echo "Sessions <input type='radio' name='use_sessions_new' value='Yes'>
            &nbsp&nbsp Http<input type='radio' name='use_sessions_new' checked 
            value='No'>\n";
   echo "</td></tr>\n";

   echo "<tr><td>Is SIDB accessible through a secure server? (this will allow
   logins with encrypted passwords, when authenticating with sessions)</td>\n";
   echo "<td>";
   if ($secure_server)
      echo "Yes <input type='radio' name='secure_server_new' checked value='Yes'>
            &nbsp&nbsp No<input type='radio' name='secure_server_new' value='No'>
            \n";
   else 
      echo "Yes <input type='radio' name='secure_server_new' value='Yes'>
            &nbsp&nbsp No<input type='radio' name='secure_server_new' checked 
            value='No'>\n";
   echo "</td></tr>\n";

   echo "<tr><td>Encrypt passwords in transit? (This will only work if you
      have a secure server and authenticate with sessions)</td>\n";
   echo "<td>";
   if ($safe_login)
      echo "Yes <input type='radio' name='safe_login_new' checked value='Yes'>
            &nbsp&nbsp No<input type='radio' name='safe_login_new' value='No'>
            \n";
   else 
      echo "Yes <input type='radio' name='safe_login_new' value='Yes'>
            &nbsp&nbsp No<input type='radio' name='safe_login_new' checked 
            value='No'>\n";
   echo "</td></tr>\n";

   echo "<tr><td>Authentification method:</td>\n";
   echo "<td><select name='auth_method_new' size=1>\n";
   if ($auth_method=="SQL")
      echo "   <option selected value='SQL'>SQL</option>\n";
   else
      echo "   <option value='SQL'>SQL</option>\n";
   if ($auth_method=="PAM")
      echo "   <option selected value='PAM'>PAM</option>\n";
   else
      echo "   <option value='PAM'>PAM</option>\n";
   if ($auth_method=="NIS")
      echo "   <option selected value='NIS'>NIS</option>\n";
   else
      echo "   <option value='NIS'>NIS</option>\n";
   echo "</select>\n";

   echo "<tr><td>(PAM authentification only:) Location of testpwd.<br> (This utility allows you to 
         authenticate users based on their password on the system)</td>\n";
   $pam_prg_length = strlen ($pam_prg)+ 10;
   echo "<td><input type='text' name='pam_prg_new' value='$pam_prg' 
         size='$pam_prg_length'></td></tr>\n";

   echo "<tr><td>(PAM and NIS authentification:)  If user is authenticated, but not known in sidb, create an account?</td>\n";
   if ($user_autocreate)
      echo "<td>Yes<input type='radio' name='user_autocreate_new' checked value='Yes'> 
         &nbsp;&nbsp;No<input type='radio' name='user_autocreate_new' value='No'></td></tr>\n";
   else
      echo "<td>Yes<input type='radio' name='user_autocreate_new' value='Yes'> 
         &nbsp;&nbsp;No<input type='radio' name='user_autocreate_new' checked value='No'></td></tr>\n";

   if (!$first_time) {
      echo "<tr><td>(PAM authentification:) When 'autocreating' a new account, to which group should the new user belong?</td>\n";
      $query ="SELECT groupid,groupname FROM groups";
      $result=do_database_query ($query);
      if ($result) {
         echo "<td><select name='auto_group_new' size=1>\n";
         $i=0;
         while ($r = db_fetch_array($result, $i)) {
            if ($auto_group==$r["groupid"])
               echo "   <option selected value='".$r["groupid"]."'>".$r["groupname"]."</option>\n";
            else
               echo "   <option value='".$r["groupid"]."'>".$r["groupname"]."</option>\n";
            $i++;
         }
         echo "</select><td></tr>\n";
      }
      else
         echo "<td>&nbsp;</td></tr>\n";
   }
   echo "<tr><td>(NIS authentification only:)  NIS domain.</td>\n";
   echo "<td><input type='text' name='nis_domain_new' value='$nis_domain' 
         size='10'></td></tr>\n";

   echo "<tr><td colspan=2 align=center><h4>Thumbnail sizes</h4></td></tr>\n";
   echo "<tr><td>Maximum size of thumbnails in pixels.</td>\n";
   echo "<td><input type='text' align='right' name='max_thumb_size_new' 
         value='$max_thumb_size' size=5>pixels</td></tr>\n";

   echo "<tr><td>Default size of thumbnails in pixels.</td>\n";
   echo "<td><input type='text' align='right' name='default_thumb_size_new' 
         value='$default_thumb_size' size=5>pixels</td></tr>\n";

   echo "<tr><td>Minimum size of thumbnails in pixels.</td>\n";
   echo "<td><input type='text' align='right' name='min_thumb_size_new' 
        value='$min_thumb_size' size=5>pixels</td></tr>\n";

   echo "<tr><td colspan=2 align=center><h4>Compression of images</h4></td></tr>\n";
   echo "<tr><td>Location of gzip (leave blank if you do not want files to be compressed)</td>\n";
   $bzip2_length = strlen ($bzip2) + 10;
   echo "<td><input type='text' name='bzip2_new' value='$bzip2' 
        size='$bzip2_length'></td></tr>\n";
   echo "<tr><td>Period after which images should be compressed (set to 0 if you do not want files to be compressed)</td>\n";
   print_exp_date_templ ($compress_delay, "<td>",false); 
   echo "</td></tr>"; 

   echo "<tr><td colspan=2 align=center><h4>Exchange of data with local filesystem</h4></td></tr>\n";
   echo "<tr><td>Where are the home directories of the users located?</td>\n";
   $people_dir_length = strlen ($people_dir) + 10;
   echo "<td><input type='text' name='people_dir_new' value='$people_dir' 
         size='$people_dir_length'></td></tr>\n";
   
   echo "<tr><td>You can set system-wide defaults for the folder names used for 
         exchange of files with SIDB, or you can let the users choose the folder
         names.<br>\n";
   echo "Do you want to impose these folder names? (recomended)</td><td>\n";
   if ($impose_folder)
      echo "Yes <input type='radio' name='impose_folder_new' checked 
            value='Yes'>&nbsp&nbsp No<input type='radio' 
            name='impose_folder_new' value='No'>\n";
   else 
      echo "Yes <input type='radio' name='impose_folder_new' value='Yes'>
            &nbsp&nbsp No<input type='radio' name='impose_folder_new' checked 
            value='No'>\n";
   echo "</td></tr>\n";
  
   echo "<tr><td>Default name for 'upload' folder.</td>\n";
   $infolder_length = strlen ($infolder) + 10;
   echo "<td><input type='text' name='infolder_new' value='$infolder' 
         size='$infolder_length'></td></tr>\n";
 
   echo "<tr><td>Default name for 'download' folder.</td>\n";
   $outfolder_length = strlen ($outfolder) + 10;
   echo "<td><input type='text' name='outfolder_new' value='$outfolder' 
         size='$outfolder_length'></td></tr>\n";

   echo "<tr><td>Maximum amount of data allowed in these folder.</td>\n";
   echo "<td><input type='text' name='max_folder_size_new' 
         value='$max_folder_size'>bytes</td></tr>\n";

   echo "<tr><td colspan=2 align=center><h4>System resource protection</h4></td></tr>\n";
   echo "<tr><td>SIDB can generate galleries with many thumbnails on the fly.
         <br>";
   echo "This puts, however, quite a strain on the resources of the system. 
         <br>";
   echo "You can therefore set a limit on the number of thumbnails in a gallery
         here.</td>\n";
   echo "<td><input type='text' name='max_in_gallery_new' 
         value='$max_in_gallery' size=4></td></tr>\n";

   echo "<tr><td>SIDB can check for duplicate images. 
         <br>";
   echo "For large files, this might take a while. 
         <br>";
   echo "Do not check for duplicates when uploading files larger than:
         </td>\n";
   echo "<td><input type='text' name='max_md5_filesize_new' 
         value='$max_md5_filesize'>bytes</td></tr>\n";

   echo "<tr><td>SIDB can generate movies of 3-D files rotating through space.
         <br>";
   echo "This puts, however, quite a strain on the resources of the system. 
         <br>";
   echo "Do not activate this unless you run huygens remotely or have a 
         multi-processor machine.</td>\n<td>";
  if ($make_rotating_movie)
      echo "Yes <input type='radio' name='make_rotating_movie_new' checked 
            value='Yes'>&nbsp&nbsp No<input type='radio' 
            name='make_rotating_movie_new' value='No'>\n";
   else 
      echo "Yes <input type='radio' name='make_rotating_movie_new' 
            value='Yes'>&nbsp&nbsp No<input type='radio' 
            name='make_rotating_movie_new' checked value='No'>\n";
   echo "</td></tr>\n";

   echo "<tr><td colspan=2 align=center><h4>Miscellaneous</h4></td></tr>\n";
   echo "<tr><td>Date format:</td>\n";
   if (!$date_format) 
      $date_format = "d-m-Y";
   if ($date_format == "d-m-Y") {
      echo "<td>d-m-y<input type='radio' name='date_format_new' value='d-m-Y' 
            checked>";
      echo " &nbsp &nbsp\n";
      echo "m-d-y<input type='radio' name='date_format_new' value='m-d-Y'>";
      echo "</td></tr>\n";
   }
   if ($date_format == "m-d-Y") {
      echo "<td>d-m-y<input type='radio' name='date_format_new' value='d-m-Y'>";
      echo " &nbsp &nbsp\n";
      echo "m-d-y<input type='radio' name='date_format_new' value='m-d-Y' 
            checked>";
      echo "</td></tr>\n";
   }

   echo "<tr><td>Upload only 2-D files(no 3-D, etc..).</td>\n"; 
   echo "<td>";
   if ($TD_only)
      echo "Yes <input type='radio' name='TD_only_new' checked 
            value='Yes'>&nbsp&nbsp No<input type='radio' 
            name='TD_only_new' value='No'>\n";
   else 
      echo "Yes <input type='radio' name='TD_only_new' 
            value='Yes'>&nbsp&nbsp No<input type='radio' 
            name='TD_only_new' checked value='No'>\n";
   echo "</td></tr>\n";

   // The new layout does not contain any logo...
   // echo "<tr><td>Image File to be used as logo in top left. Use a relative path (for instance: 'images/svilogo.gif').</td>\n";
   // echo "<td>
   echo "<input type='hidden' name='logo_new' value='$logo'></td></tr>\n";
   // echo "<tr><td>Link for logo.  Use a valid URL (for instance: http://sidb.sf.net)</td>\n";
   // echo "<td>
   echo "<input type='hidden' name='logo_link_new' value='$logo_link'></td></tr>\n";

   // echo "<tr><td>Desired width of logo.</td>\n";
   // echo "<td>
   echo "<input type='hidden' name='logo_x_size_new' value='$logo_x_size'></td></tr>\n";

   // echo "<tr><td>Desired height of logo.</td>\n";
   // echo "<td>
   echo "<input type='hidden' name='logo_y_size_new' value='$logo_y_size'></td></tr>\n";

   echo "<tr><td>File to log SQL statements. Leave empty except when debugging. 
         Otherwise, try $localdir/SQL_log.txt</td>\n";
   echo "<td><input type='text' name='SQL_log_new' value='$SQL_log'></td></tr>
          \n";

   echo "<tr><td>SIDB users that will generate a notification when uploading.
         </td>";
   echo "<td><input type='text' name='emailNotifyList_new' 
         value='$emailNotifyList'></td></tr>";
   echo "<tr><td>Destination of that automatic notification.</td>";
   echo "<td><input type='text' name='emailNotifyAddress_new' 
         value='$emailNotifyAddress'></td></tr>";
   echo "<tr><td>Sender of that notification will appear as: </td>";
   echo "<td><input type='text' name='emailSenderAddress_new' 
         value='$emailSenderAddress'></td></tr>";

   echo "<tr><td colspan=2 align='center'><input align=center type=submit 
         name=action value=submit></td></tr>\n";  

   if ($first_time)
      echo "<tr><td colspan=2 align='center'>After submitting this form, 
            please login as <b>sysadmin</b> using the password <b>$newsys_pass
            </b>.</td></tr>\n";

   echo "</table>\n";
   echo "</form>";
}  

////
// uncompresses all compressed files, this migh break in a fresh install
function uncompress_all() {
   $query = "SELECT imageid FROM images WHERE compressed='t'";
   $result = do_database_query ($query);
   $i=0;
   while ($r = db_fetch_array($result, $i)) {
      $imageid = $r["imageid"];
      uncompress($imageid);
      $query = "UPDATE images SET compressed='f' WHERE imageid=$imageid";
      do_database_query ($query);
      $i++;
   }
}

////
// Processes the submitted form
function process_config_form () {
   global $archive_dir, $thumbnail_dir, $include_dir, $IMAGE_TOOLS;
   global $localdir, $image_tool, $convert_path, $movie_tool;
   global $pam_prg, $use_imagej, $max_size_for_convert, $max_size_for_movie;
   global $use_sessions, $secure_server, $safe_login;
   global $max_thumb_size, $default_thumb_size, $min_thumb_size;
   global $people_dir, $impose_folder, $infolder, $outfolder;
   global $max_folder_size, $max_in_gallery, $PHP_SELF, $SQL_log;
   global $webdaemon, $HTTP_POST_VARS, $version, $version_code;
   global $make_rotating_movie, $date_format, $sidb_root;
   global $bzip2, $compress_delay;
   global $auth_method,$nis_domain,$user_autocreate,$auto_group,$max_md5_filesize;
   global $TD_only,$logo,$logo_x_size,$logo_y_size,$logo_link;
   global $emailNotifyList, $emailNotifyAddress, $emailSenderAddress;


   extract ($HTTP_POST_VARS, EXTR_PREFIX_SKIP, "post_vars");   

   // read $config_inc into an array
   $configarray = @file ("$localdir/includes/config_inc.php");
   if (!$configarray) {
      echo "<h3>Could not read the file: $localdir/includes/config_inc.php<br>";
      echo "Please make sure it exists and can be read by the webmaster.</h3>";
      exit();
   }

   //set compress_delay right:
   $compress_delay_new = $exp_number * $exp_unit;

   if ($version < $version_code) {
      // do whatever is needed

      if($version < 0.14) {
         // move index.htm to index_inc.htm, and adjust the line in index.php
         $indexarray = file ("$localdir/index.php");
         $size = sizeof ($indexarray);
         if ($indexarray) {
            for ($i=0; $i < sizeof ($indexarray); $i++) {
               if (strstr ($indexarray[$i], "include (\"index.htm\")")) {
                  $indexarray[$i] = '   include ("index_inc.htm");'."\n";
               }
            }
            // write the changed file
            if (!(@chmod ("$localdir/index.php", 0700))) {
               error_not_owned_error ($webdaemon, "$localdir", "index.php");
               exit ();
            }
            $findex = fopen ("$localdir/index.php", "w");
            for ($i = 0; $i<sizeof($indexarray); $i++) 
               fwrite ($findex, $indexarray[$i]);
            fclose ($findex); 
            $result = `mv $localdir/index.htm $localdir/index_inc.htm`;
         }
         else echo "Could not read the file $localdir/index.php<br>";
      }

      if ($version < 0.16003) {
         $query = "ALTER TABLE images ADD COLUMN fs_thumb smallint";
         do_database_query ($query);
         $query = "ALTER TABLE template ADD COLUMN fs_thumb smallint";
         do_database_query ($query);
         echo "Version smaller than 0.16003.<br>";
      }
      
      if ($version < 0.16005) {
         $query = "ALTER TABLE images ADD COLUMN exp_date integer";
         do_database_query ($query);
         $query = "ALTER TABLE template ADD COLUMN exp_period integer";
         do_database_query ($query);
         $query = "ALTER TABLE images ADD COLUMN exp_mail_date integer";
         do_database_query ($query);
      }

      if ($version < 0.16006) {
         $query = "UPDATE translate SET translation='Template name' WHERE 
                   columnname='templatename';";
         do_database_query ($query);
      }

      if ($version < 0.17002) {
         do_database_query ("BEGIN WORK");
         $query = "UPDATE translate SET translation = 'Me:' WHERE
                   columnname = 'private';";
         if (!do_database_query ($query)) $wrong= true;
         $query = "UPDATE translate SET translation = 'My group:' WHERE
                   columnname = 'restrict_to_group';";
         if (!do_database_query ($query)) $wrong= true;
         $query = "INSERT INTO translate (translation,columnname)
                   VALUES ('Everyone:','everyone');";
         if (!do_database_query ($query)) $wrong= true;
         // Create table projects
         $query = "CREATE TABLE projects (projectid serial, name varchar,
                   description varchar, userid int4, rights int2);";
         if (!do_database_query ($query)) $wrong= true;
         $query = "CREATE TABLE projectximages (projectid int4 NOT NULL,
                   imageid int4 NOT NULL);";
         if (!do_database_query ($query)) $wrong= true;
         $query = "CREATE TABLE projectxprojects (motherid int4 NOT NULL,
                   daughterid int4 NOT NULL);";
         if (!do_database_query ($query)) $wrong= true;
         if ($wrong) {
            echo "Problems updating the database;(.<br>";
            do_database_query ("ROLLBACK WORK");
         }
         else {
            do_database_query ("COMMIT WORK");
            `mkdir $thumbnail_dir/projects`;
            `mkdir $thumbnail_dir/projects/big`;
            `mkdir $thumbnail_dir/projects/small`;
            if (! (file_exists("$thumbnail_dir/projects") &&
                   file_exists ("$thumbnail_dir/projects")) )
               echo "Problems creating subdirectories in $thumbnail_dir.<br>";
         }
      }
      if ($version < 0.17005) {
         $query = "CREATE TABLE projectxtemplate (projectid int4 NOT NULL,
                   templateid int4 NOT NULL);";
         do_database_query ($query);
      }
      if ($version < 0.1801) {
         $query = "ALTER TABLE template ADD COLUMN filetype varchar";
         do_database_query ($query);
      }
      if ($version < 0.1802) {
         do_database_query("BEGIN WORK");
         $query = "ALTER TABLE filetypes ADD COLUMN sortid int2";
         do_database_query ($query);
         $query = "INSERT INTO filetypes (name,sortid)
                   VALUES ('multipage','10');";
         if (!do_database_query ($query)) $wrong= true;
         $query = "UPDATE filetypes SET sortid = '1' WHERE name='2-D'";
         if (!do_database_query ($query)) $wrong= true;
         $query = "UPDATE filetypes SET sortid = '20' WHERE name='3-D'";
         if (!do_database_query ($query)) $wrong= true;
         $query = "UPDATE filetypes SET sortid = '21' WHERE name='ICS'";
         if (!do_database_query ($query)) $wrong= true;
         $query = "UPDATE filetypes SET sortid = '30' WHERE name='other'";
         if (!do_database_query ($query)) $wrong= true;
	 if ($wrong) {
            echo "Problems updating the database!.<br>";
            do_database_query ("ROLLBACK WORK");
         }
         else
            do_database_query ("COMMIT WORK");
      }
      if ($version < 0.1805) {
         changeline ($configarray, "use_sessions", true);
         changeline ($configarray, "secure_server", false);
         changeline ($configarray, "safe_login", false);
         changeline ($configarray, "pam_prg", false);
      }
      if ($version < 0.1806) {
         $query = "ALTER TABLE images ADD COLUMN compressed boolean";
         changeline ($configarray, "bzip2", which("gzip"));
         $month=mktime(0,0,0,2,1,2000)-mktime(0,0,0,1,1,2000);
         changeline ($configarray, "compress_delay", $month);
         do_database_query ($query);
      }
      if ($version < 0.191) {
         $query="CREATE TABLE imagextrust (
            imageid int4 NOT NULL,
            userid int4 NOT NULL)";
         do_database_query ($query);
         // move values in trust into this new table
         $query="SELECT imageid,trust FROM images WHERE trust>-1";
         $r=db_exec($query);
         $nr=db_numrows($r);
         for ($i=0;$i<$nr;$i++) {
           $row=db_fetch_array($r,$i);
           $query="INSERT INTO imagextrust (imageid,userid) VALUES
                   (".$row["imageid"].",".$row["trust"].")";
           db_exec($query);
         }
      } 
      if ($version < 0.202) {
         $query="ALTER TABLE images ADD COLUMN md5sum text";
         do_database_query($query);
      }
      if ($version < 0.203) {
         $query="ALTER TABLE images ADD COLUMN pinsize float8";
         do_database_query($query);
         $query="ALTER TABLE images ADD COLUMN pinspacing float8";
         do_database_query($query);
         $query="ALTER TABLE template ADD COLUMN pinsize float8";
         do_database_query($query);
         $query="ALTER TABLE template ADD COLUMN pinspacing float8";
         do_database_query($query);
      }

	if ($version < 0.40) {

           $include_array = file ("include.php");                     
           if ($include_array) {
             $finclude = fopen ("include.php", "w");                 
               for ($i = 0; $i<sizeof($include_array); $i++) {
                 if (strstr ($include_array[$i], "config_inc"))  {
                  $include_array[$i]="require(\"./includes/classes.php\");\n";
                  $include_array[$i].="require(\"./includes/config_inc.php\");\n";
                 }
                 fwrite ($finclude, $include_array[$i]);              
               } 
           fclose ($finclude);
           } 

           $query = "CREATE TABLE images_new
            (
                imageid INT4 DEFAULT nextval('images_imageid_seq'::TEXT) NOT NULL,
                filename TEXT NOT NULL,
                description TEXT,
                userid INT4 NOT NULL,
                private BOOL DEFAULT 't'::BOOL,
                restrict_to_group BOOL,
                everyone BOOL,
                trust INT4,
                date_time INT4 NOT NULL,
                exp_date INT4,
                exp_mail_date INT4,
                sensortype CHARACTER VARYING,
                channels INT2,
                numaperture FLOAT8,
                refrinxmedium FLOAT8,
                refrinxlensmedium FLOAT8,
                filetype CHARACTER VARYING,
                filetype_extension CHARACTER VARYING,
                raw_data BOOL,
                microscope_type INT2,
                x_thumb INT2,
                y_thumb INT2,
                fs_thumb INT2,
                x_nm FLOAT8,
                y_nm FLOAT8,
                z_nm FLOAT8,
                t_sec FLOAT8,
                thumb_nr INT2,
                filesize INT8,
                channel_data TEXT,
                mime TEXT,
                settings TEXT,
                x_pixel smallINT,
                y_pixel smallINT,
                z_pixel smallINT,
                compressed BOOL,
                pinsize FLOAT8,
                pinspacing FLOAT8,
                md5sum TEXT
            )
            ";

           do_database_query($query);

           $query = "INSERT INTO images_new (imageid, description, userid, private, restrict_to_group, everyone, trust, date_time,  exp_date, exp_mail_date, sensortype, channels, numaperture, refrinxmedium, refrinxlensmedium, filetype, filetype_extension, raw_data, microscope_type, x_thumb, y_thumb, fs_thumb, x_nm, y_nm, z_nm, t_sec, thumb_nr, filesize, channel_data, mime, settings, x_pixel, y_pixel, z_pixel, compressed, pinsize, pinspacing, md5sum, filename) ".
            "SELECT  imageid, description, userid, private, restrict_to_group, everyone, trust, date_time,  exp_date, exp_mail_date, sensortype, channels, numaperture, refrinxmedium, refrinxlensmedium, filetype, filetype_extension, raw_data, microscope_type, x_thumb, y_thumb, fs_thumb, x_nm, y_nm, z_nm, t_sec, thumb_nr, filesize, channel_data, mime, settings, x_pixel, y_pixel, z_pixel, compressed, pinsize, pinspacing, md5sum, filename ".
           "FROM images;";

           do_database_query($query);

           $query = "DROP TABLE images;";
           do_database_query($query);

           $query = "ALTER TABLE images_new RENAME TO images;";
           do_database_query($query);

           // Increase list of avaliable colours for channels
           $query="INSERT INTO colors (colorid,color_hex,color_text,color_sort,color_string) VALUES ('9', '#0ff', 'cyan', '3200', '0 255 255')";
           do_database_query($query);
           $query="INSERT INTO colors (colorid,color_hex,color_text,color_sort,color_string) VALUES ('10', '#f0f', 'magenta', '3400', '255 0 255')";
           do_database_query($query);
           $query="INSERT INTO colors (colorid,color_hex,color_text,color_sort,color_string) VALUES ('11', '#ff0', 'yellow', '3600', '255 255 0')";
           do_database_query($query);
           $query="ALTER TABLE microscopes ADD COLUMN huygens_scope text";
           do_database_query($query);
           $query="UPDATE microscopes SET huygens_scope = 'confocal' WHERE lower(microscope) LIKE 'conf%'";
           do_database_query($query);
           $query="UPDATE microscopes SET huygens_scope = 'widefield' WHERE lower(microscope) LIKE 'wide%'";
           do_database_query($query);
           $query="UPDATE microscopes SET huygens_scope = '4Pi' WHERE lower(microscope) LIKE '4pi%'";
           do_database_query($query);
           $query="UPDATE microscopes SET huygens_scope = 'generic' WHERE lower(microscope) LIKE 'gener%'";
           do_database_query($query);
          $query="INSERT INTO microscopes (microscope, huygens_scope) VALUES ('Nipkow disk', 'nipkow')";
           do_database_query($query);

           /*
           $query="ALTER TABLE images ADD COLUMN filename_long text";
           do_database_query($query);
           $query="UPDATE images SET filename_long=filename";
           do_database_query($query);
           $query="ALTER TABLE images RENAME filename TO filename_short";
           do_database_query($query);
           $query="ALTER TABLE images RENAME filename_long TO filename";
           do_database_query($query);
           $query="ALTER TABLE images DROP filename_short";
           do_database_query($query);
           $query="ALTER TABLE images ALTER filename SET NOT NULL";
           do_database_query($query);
           */




	    /*
	     * Version 0.40 has the fix of the thumbnails created
	     * with Huygens.
	     *
	     * Huygens can read .ids and .ids.gz files with a .ics
	     * file. Uploading both .ids and .ids.gz works but SIDB
	     * internally only uses the extension .ids, failing the
	     * creation of thumbnails when a .ids.gz file is 
	     * uploaded. This functionality here will gzip all .ids
	     * files in the archive.
	     *
	     * When .ids.gz is uploaded, nothing is done with the
	     * file. When .ids is uploaded, it is gzipped. So in
	     * stead of .ids, now .ids.gz is used internally.
	     * This saves both diskspace and bandwidth.
	     *
	     * Get all .ics images and gzip all .ids images with
	     * them.
	     */
	    $db_query = "SELECT imageid, filename FROM images WHERE filetype='ICS'";
	    $db_result = do_database_query($db_query);
	    $db_rows = db_NumRows($db_result);
	    print("<p>Gzipping all database IDS files");
	    for ($rows=0; $rows < $db_rows; $rows++)
	    {
		$db_array = db_Fetch_Array($db_result, $row);
		$filename_ics = $db_array["imageid"]."_".$db_array["filename"];
		$filename_ids = str_replace(".ics", ".ids", $filename_ics);
		print("<p>");
		print("Found ICS filename: $filename_ics. ");
                if(file_exists("$archive_dir/$filename_ids")) {

   		   print("Current IDS filename: $filename_ids. ");
		   print("<p>");
		   print("Gzipping: $bzip2 '$archive_dir/$filename_ids' ");
		   if (file_exists($bzip2)) exec("$bzip2 '$archive_dir/$filename_ids'");
                   else print("<p>Zip tool not found, compress ids files manually!");
                }
	   }
	   print("<p>Done gzipping all IDS files. ");
	}

      if ($version < 0.41) {

         /* A bug in 0.40, if installed from scratch, did not create column
         *  huygens_scope in table microscopes. Correct it now! */

	 $query = "SELECT huygens_scope FROM microscopes WHERE microscopeid=1";
         $db_result = do_database_query($query);
         $db_rows = db_NumRows($db_result);
         if ($db_rows != 1) {
	   echo "<br>Do not worry for that error, it means that the database ".
           "has to be updated. I'll do it now...<br>";
           $query="ALTER TABLE microscopes ADD COLUMN huygens_scope text";
           do_database_query($query);
           $query="UPDATE microscopes SET huygens_scope = 'confocal' WHERE lower(microscope) LIKE 'conf%'";
           do_database_query($query);
           $query="UPDATE microscopes SET huygens_scope = 'widefield' WHERE lower(microscope) LIKE 'wide%'";
           do_database_query($query);
           $query="UPDATE microscopes SET huygens_scope = '4Pi' WHERE lower(microscope) LIKE '4pi%'";
           do_database_query($query);
           $query="UPDATE microscopes SET huygens_scope = 'generic' WHERE lower(microscope) LIKE 'gener%'";
           do_database_query($query);
          $query="INSERT INTO microscopes (microscope, huygens_scope) VALUES ('Nipkow disk', 'nipkow')";
           do_database_query($query);
         }
     }


      changeline ($configarray, "version", $version_code);
      echo "<p>SIDB has been updated to version $version_code.<br>";
   }



   // change the variables according to the input 

   // postgres before 7.1 had a tuple size of about 8100, we need to know:
   exec ("psql -V", $pversion);
   $pverarray=explode(" ",$pversion[0]);
   $pver=end($pverarray);
   $pver=substr($pver,0,3);
   if ($pver<7.1)
      $old_postgres_new=1;
   else
      $old_postgres_new=0;
   if ($old_postgres_new <> $old_postgres)
      changeline ($configarray, "old_postgres", $old_postgres_new);
   
   if ($archive_dir_new) 
      if (movedir ($archive_dir_new, $archive_dir, "archive", $webdaemon))
         changeline ($configarray, "archive_dir", $archive_dir_new);
   if ($thumbnail_dir_new) 
      if (movedir ($thumbnail_dir_new, $thumbnail_dir, "thumbnail", $webdaemon))
         changeline ($configarray, "thumbnail_dir", $thumbnail_dir_new);
   if ($convert_path_new && ($convert_path_new <> $convert_path))
      changeline ($configarray, "convert_path", $convert_path_new);
   $max_size_for_convert_new = (int) $max_size_for_convert_new;
   if ($max_size_for_convert_new && 
         ($max_size_for_convert_new <> $max_size_for_convert))
      changeline ($configarray, "max_size_for_convert", $max_size_for_convert_new);
   if ($IMAGE_TOOLS_new == "Yes") 
      $IMAGE_TOOLS_new = true; 
   else 
      $IMAGE_TOOLS_new = false;
   if ($IMAGE_TOOLS <> $IMAGE_TOOLS_new)
      changeline ($configarray, "IMAGE_TOOLS", $IMAGE_TOOLS_new);
   if ($image_tool_new && ($image_tool_new <> $image_tool))
      changeline ($configarray, "image_tool", $image_tool_new);

   if ($use_sessions_new == "Yes")
      $use_sessions_new = true;
   else
      $use_sessions_new = false;
   if ($use_sessions_new <> $use_sessions) {
      changeline ($configarray, "use_sessions", $use_sessions_new);
      // we also need to adjust index.php
      $indexarray = @file ("$localdir/index.php");
      if (!$indexarray) {
       echo "<h3>Could not read the file: $localdir/index.php<br>";
       echo "Please make sure it exists and can be read by the webmaster.</h3>";
      }
      changeline ($indexarray, "use_sessions", $use_sessions_new);
      // write the changed file
      if (!(@chmod ("$localdir/index.php", 0700))) {
         error_not_owned_error ($webdaemon,"$localdir/", "index.php");
      }
      $fconfig = fopen ("$localdir/index.php", "w");
      for ($i = 0; $i<sizeof($indexarray); $i++) 
         fwrite ($fconfig, $indexarray[$i]);
      fclose ($fconfig); 
   }

   if ($secure_server_new == "Yes")
      $secure_server_new = true;
   else
      $secure_server_new = false;
   if ($secure_server_new <> $secure_server)
      changeline ($configarray, "secure_server", $secure_server_new);
   if ($safe_login_new == "Yes")
      $safe_login_new = true;
   else
      $safe_login_new = false;
   if ($safe_login_new <> $safe_login)
      changeline ($configarray, "safe_login", $safe_login_new);
   if ($auth_method_new <> $auth_method)
      changeline ($configarray,"auth_method", $auth_method_new);
   if ($nis_domain_new <> $nis_domain)
      changeline ($configarray,"nis_domain", $nis_domain_new);
   if ($user_autocreate_new <> $user_autocreate)
      changeline ($configarray,"user_autocreate", $user_autocreate_new);
   if ($auto_group_new <> $auto_group)
      changeline ($configarray,"auto_group", $auto_group_new);

   if ($pam_prg_new <> $pam_prg)
      changeline ($configarray, "pam_prg", $pam_prg_new);

   if ($bzip2_new <> $bzip2) {
      // for now we only allow gzip
      if (!$bzip2_new || basename($bzip2_new)=="gzip") {
         // if we inactivate compression, we first should decompress all
         if (!$bzip2_new)
            uncompress_all(); 
         changeline ($configarray, "bzip2", $bzip2_new);
      }
   }
   if ($compress_delay_new <> $compress_delay) {
      // if we inactivate compression, we first should decompress all
      if (!$compress_delay_new)
         uncompress_all(); 
      changeline ($configarray, "compress_delay", $compress_delay_new);
   }
   if ($movie_tool_new <> $movie_tool)
      if ( ( strstr ($movie_tool_new, "convert") ||
             strstr ($movie_tool_new, "mpeg2encode") ||
             strstr ($movie_tool_new, "ffmpeg") ) &&
           is_readable ($movie_tool_new) )  
         changeline ($configarray, "movie_tool", $movie_tool_new);
      else
         echo "<b>$movie_tool_new</b> is not a valid movietool.</br>";
   $max_size_for_movie_new = (int) $max_size_for_movie_new;
   if ($max_size_for_movie_new && 
         ($max_size_for_movie_new <> $max_size_for_movie))
      changeline ($configarray, "max_size_for_movie", $max_size_for_movie_new);
   if ($use_imagej_new == "Yes") 
      $use_imagej_new = true; 
   else 
      $use_imagej_new = false;
   if ($use_imagej <> $use_imagej_new)
      changeline ($configarray, "use_imagej", $use_imagej_new);
   $max_thumb_size_new = (int) $max_thumb_size_new;
   if ($max_thumb_size_new && ($max_thumb_size_new <> $max_thumb_size))
      changeline ($configarray, "max_thumb_size", $max_thumb_size_new);
   $default_thumb_size_new = (int) $default_thumb_size_new;
   if ($default_thumb_size_new && 
         ($default_thumb_size_new <> $default_thumb_size))
      changeline ($configarray, "default_thumb_size", $default_thumb_size_new);
   $min_thumb_size_new = (int) $min_thumb_size_new;
   if ($min_thumb_size_new && ($min_thumb_size_new <> $min_thumb_size))
      changeline ($configarray, "min_thumb_size", $min_thumb_size_new);
   if (is_readable ($people_dir_new) && ($people_dir_new <> $people_dir))
      changeline ($configarray, "people_dir", $people_dir_new);
   if ($impose_folder_new == "Yes") 
      $impose_folder_new = true; 
   else 
      $impose_folder_new = false;
   if ($impose_folder <> $impose_folder_new)
      changeline ($configarray, "impose_folder", $impose_folder_new);
   if ($infolder_new && ($infolder_new <> $infolder))
      changeline ($configarray, "infolder", $infolder_new);
   if ($outfolder_new && ($outfolder_new <> $outfolder))
      changeline ($configarray, "outfolder", $outfolder_new);
   $max_folder_size_new = $max_folder_size_new;
   if ($max_folder_size_new && ($max_folder_size_new <> $max_folder_size))
      changeline ($configarray, "max_folder_size", $max_folder_size_new);
   $max_in_gallery_new = (int) $max_in_gallery_new;
   if ($max_in_gallery_new && ($max_in_gallery_new <> $max_in_gallery))
      changeline ($configarray, "max_in_gallery", $max_in_gallery_new);
   $max_md5_filesize_new = (int) $max_md5_filesize_new;
   if ($max_md5_filesize_new && ($max_md5_filesize_new <> $max_md5_filesize))
      changeline ($configarray, "max_md5_filesize", $max_md5_filesize_new);
   if ($make_rotating_movie_new == "Yes") 
      $make_rotating_movie_new = true; 
   else 
      $make_rotating_movie_new = false;
   if ($make_rotating_movie <> $make_rotating_movie_new)
     changeline ($configarray, "make_rotating_movie", $make_rotating_movie_new);
   if ($date_format_new <> $date_format)
      changeline ($configarray, "date_format", $date_format_new);
   if ($TD_only_new == "Yes") 
      $TD_only_new = true; 
   else 
      $TD_only_new = false;
   if ($TD_only <> $TD_only_new)
      changeline ($configarray, "TD_only", $TD_only_new);
   if ($logo <> $logo_new)
      changeline ($configarray, "logo", $logo_new);
   if ($logo_x_size <> $logo_x_size_new)
      changeline ($configarray, "logo_x_size", $logo_x_size_new);
   if ($logo_y_size <> $logo_y_size_new)
      changeline ($configarray, "logo_y_size", $logo_y_size_new);
   if ($logo_link <> $logo_link_new)
      changeline ($configarray, "logo_link", $logo_link_new);
   if ($SQL_log_new <> $SQL_log)
      changeline ($configarray, "SQL_log", $SQL_log_new);
   if ($emailNotifyList_new <> $emailNotifyList)
      changeline ($configarray, "emailNotifyList", $emailNotifyList_new);
   if ($emailNotifyAddress_new <> $emailNotifyAddress)
      changeline ($configarray, "emailNotifyAddress", $emailNotifyAddress_new);
   if ($emailSenderAddress_new <> $emailSenderAddress)
      changeline ($configarray, "emailSenderAddress", $emailSenderAddress_new);
   $sidb_root_new = sidb_root();
   if ($sidb_root_new <> $sidb_root)
      changeline ($configarray, "sidb_root", $sidb_root_new); 

   // write the changed file
   if (!(@chmod ("$localdir/includes/config_inc.php", 0700))) {
      error_not_owned_error ($webdaemon,"$localdir/includes", "config_inc.php");
      exit ();
   }
   $fconfig = fopen ("$localdir/includes/config_inc.php", "w");
   for ($i = 0; $i<sizeof($configarray); $i++) 
      fwrite ($fconfig, $configarray[$i]);
   fclose ($fconfig); 
   echo "<h3>Wrote the new values into the database</h3>";

} 

////
// Function that moves files from config_templates to their proper location
function move_config_file ($localdir, $the_file, $webdaemon) {

  if (!file_exists ("$localdir/$the_file"))
      if (file_exists ("$localdir/config_templates/$the_file")) {
         if (!rename ("$localdir/config_templates/$the_file", 
                      "$localdir/$the_file")) {
            echo "<h3>Could not write to directory $localdir.  Please check 
                  that it and all the files therein are owned by $webdaemon 
                  and that they are writeable.  
                  Then try again.</h3>";
            exit ();
         }
     }
     else {
         echo "<h3 color='red'>Fatal error.  The file $the_file disappeared.  
                Try re-installing the sidb distribution.</h3>";
         exit ();
      }
}

/************************* start of script body ****************************/


$php_ini = get_cfg_var ("cfg_file_path");

// check for safe mode
$test_exec = exec ("ls");
if (!$test_exec)
   if ($safe_mode = ini_get("safe_mode")) {
      echo "<h3>PHP appears to be working in safe mode, and sidb can not execute 
            some of its essential functions.  Please fix this first (possibly 
            by editing the configuration file $php_ini).";
      exit ();
   }

// check whether php supports postgresql
if (!function_exists ("pg_connect")) { 
   echo "<h3>The version of php you are using appears not to support Postgresql.  Please re-compile php with support for postgresql, or download and install a php rmp with Postgresql, or download and install an rpm providing a module with Postgresql support for php.";
   exit ();
}

// check whether postmaster is running
if (checkd("postmaster"))
{
    $postmaster = TRUE;
}
elseif (checkd("postgres"))
{
    $postmaster = TRUE;
}
else
{
    print("<h3>");
    print("Postmaster (the daemon behind PostgresQL) is not running.");
    print("Fix this first and then return here");
    print("</h3>");
    exit ();
}

$localdir = exec ("pwd");
$webdaemon = exec ("whoami");

move_config_file ($localdir, "include.php", $webdaemon);
move_config_file ($localdir, "index.php", $webdaemon);
move_config_file ($localdir, "includes/config_inc.php", $webdaemon);
move_config_file ($localdir, "index_inc.htm", $webdaemon);

// read $config_inc into an array
$configarray = @file ("$localdir/includes/config_inc.php");
if (!$configarray) {
   echo "<h3>Could not read the file: $localdir/includes/config_inc.php<br>";
   echo "Please make sure it exists and can be read by the webmaster.</h3>";
   exit();
}

for ($i = 0; $i< sizeof($configarray); $i ++) {
   $line = $configarray[$i];
   if ($line[0] == "$") {
      $value = trim (strtok ($line, "$="));
      // it seems that some php version behave differently....
      if (strlen($value)<2)
         $value = trim (strtok ("$="));
      ${$value} = trim (strtok (";"));
      ${$value} = str_replace ("\"", "", ${$value});
   }
}

if ($version<0.40) {
      $configarray=str_replace("auth_inc","class-auth",$configarray);

      $fconfig = fopen ("$localdir/includes/config_inc.php", "w");
      for ($i = 0; $i<sizeof($configarray); $i++)
         fwrite ($fconfig, $configarray[$i]);
      fclose ($fconfig);
}


// try to setup a connection to the database;
$db_conn = @pg_connect ("dbname=$db_name user=$db_user password=$db_pass");
if ($db_conn) {
  // echo "database connection made<br>";
   @pg_close ($db_conn);
   $db_present = true;
   require ("$localdir/includes/classes.php");
   require ("$localdir/includes/config_inc.php");
   //changeline ($configarray, "db_name", $db_name);
   Auth::allowonly ($G_SUPER);
   if ( ($action <>"submit") && ($action <> "Vacuum") ) {
      $title = "sidb system settings";
      printheader ();
      config_form (false);
      printfooter ();
   }
}

elseif ($action <>"go") {
   // present a form with last chance to change databasename etc..
   echo "<form enctype='multipart/form-data' method='post' action=$PHP_SELF>\n";
   echo "<h3>A new database for SIDB is about to be created. ";
   echo "Please review the following parameters carefully</h3>";
   echo "<hr>";
   echo "<table border=0>\n";
   echo "<tr><td>Database name </td>";
   echo "<td><input type=text name=newdb_name value=$db_name></td></tr>\n";
   echo "<tr><td>Database user (valid postgresql user)</td>";
   echo "<td><input type=text name=newdb_user value=$db_user></td></tr>\n";
   echo "<tr><td>Database user password (only needed when you <br>want to 
         secure postgresQL access later on)</td>";
   echo "<td><input type=text name=newdb_pass value=$db_pass></td></tr>\n";
   echo "<tr><td>Sysadmin password (for SIDB access), <b>change this</b></td>";
   echo "<td><input type=text name=newsys_pass value=psswd></td></tr>\n";
   if (! (which ("createdb") && which ("psql") && which ("dropdb"))) {
      echo "<tr><td>Path to postgresql binaries</td>\n";
      echo "<td><input type=text name=postgres_path></td></tr>\n";
   } 
   echo "</table>";
   echo "<hr>";
   echo "<h4>After submitting this form, a new database will be created, 
          and you will soon be asked to login as sysadmin, 
          using the password above.</h4>";
   echo "<input align=center type=submit name=action value=go>\n";   
   echo "</form>\n";
}

elseif ($action=="go") {
   // write the new variables back to config_inc.php
   if ($newdb_name) $db_name = $newdb_name;
   if ($newdb_user) $db_user = $newdb_user;
   if ($newdb_pass) $db_pass = $newdb_pass;
   changeline ($configarray, "db_name", $db_name);
   changeline ($configarray, "db_user", $db_user);
   changeline ($configarray, "db_pass", $db_pass);
   if (!(@chmod ("$localdir/includes/config_inc.php", 0700))) {
      error_not_owned_error ($webdaemon,"$localdir/includes","config_inc.php");
      exit ();
   }
   $fconfig = fopen ("$localdir/includes/config_inc.php", "w");
   for ($i = 0; $i<sizeof($configarray); $i++) 
      fwrite ($fconfig, $configarray[$i]);
   fclose ($fconfig); 
        
   // check the right on thumbnail and archive dirs
   if (!@chmod ("archive", 0700)) {
      error_not_owned_error ($webdaemon, $localdir, "archive");
      exit ();
   }
   if (!@chmod ("thumbnails", 0700)) {
      error_not_owned_error ($webdaemon, $localdir, "thumbnails");
      exit ();
   }
    
   // set the links in index.php and include.php rigt.
   if (!(@chmod ("include.php", 0700))) {
      error_not_owned_error ($webdaemon, $localdir, "include.php");
      exit ();
   }
   // This shouldn't be necessary, with relative paths ./includes

//   $include_array = file ("include.php");
//   if ($include_array) {
//      $finclude = fopen ("include.php", "w");
//      for ($i = 0; $i<sizeof($include_array); $i++) {
//         if (strstr ($include_array[$i], "config_inc"))  {
//         $include_array[$i]="require(\"$localdir/includes/config_inc.php\");\n";
//         }
//         if (strstr ($include_array[$i], "classes"))  {
//         $include_array[$i]="require(\"$localdir/includes/classes.php\");\n"; 
//         }
//         fwrite ($finclude, $include_array[$i]);
//      }
//      fclose ($finclude);
//   }
   if (!(@chmod ("index.php", 0700))) {
      error_not_owned_error ($webdaemon, $localdir, "index.php");
      exit ();
   }
   // This shouldn't be necessary, with relative paths ./includes
   //$index_array = file ("index.php");
   //if ($index_array) {
   //  $findex = fopen ("index.php", "w");
   //   for ($i = 0; $i<sizeof($index_array); $i++) {
   //      if (strstr ($index_array[$i], "require"))  {
   //         $index_array[$i]="require (\"$localdir/includes/init_inc.php\");\n";
   //      }
   //      fwrite ($findex, $index_array[$i]);
   //   }
   //   fclose ($findex);
   //}
   
   // initialize the database

   // find the postresql binaries
   if ($postgres_path) {
      $strlength = strlen ($postgres_path);
      // check if the path ended with a slash
      if ($postgres_path[$strlength] <> "/")
         $postgres_path .= "/";
      $createdb = $postgres_path . "createdb";
      $dropdb = $postgres_path . "dropdb";
      $psql = $postgres_path . "psql";
   }
   else {
      $createdb = which ("createdb");
      $dropdb = which ("dropdb");
      $psql = which ("psql");
   }

   if (!($createdb && $dropdb && $psql) ) {
      echo "Could not find the postgres binaries 
            (createdb, dropdb, and/or psql)<br>";
      echo "Please install postgres, or provide a path to those binaries.";
      exit ();
   }
   
   $command = "$createdb -U $db_user $db_name";
   $manualcommand = "$createdb -U $db_user -W $db_name";
   $db_result = exec ($command);
   // test the new database by making a connection
   $db_conn = @pg_connect ("dbname=$db_name user=$db_user password=$db_pass");
   if ($db_conn) {
      
      // read database.pgsql 
      $dbarray = @file ("$localdir/database.pgsql");
      if (!$dbarray) {
         echo "<h3>Could not read the file: $localdir/database.pgsql<br>";
         echo "Please make sure it exists and can be read by the webmaster.
               </h3>";
         pg_close ($db_conn);
         exec ("$dropdb -U $db_user $db_name");
         exit();
      }
      /*
      // change the first line
      $dbarray[0] = '\connect - '.$db_user."\n";
      // and write back to the database file
      if (!(@chmod ("$localdir/database.pgsql", 0700) ) ) {
         error_not_owned_error ($webdaemon, "$localdir", "database.pgsql");
         pg_close ($db_conn);
         exec ("$dropdb -U $db_user $db_name");
         exit ();
      }
      $fdb = fopen ("$localdir/database.pgsql", "w");
      for ($i=0; $i < sizeof ($dbarray); $i++)
         fwrite ($fdb, $dbarray[$i]);
      fclose ($fdb);
      */

      // fill the database
      $db_file = "$localdir/database.pgsql";
      if ($db_file) {
         $result = exec ("$psql -U $db_user $db_name < $db_file");
         // try to set the new passworda
         if (!$newsys_pass) $newsys_pass="psswd";
         $pwd_md5=md5($newsys_pass);
         $query ="UPDATE users SET pwd='$pwd_md5' WHERE login='sysadmin';";
         if (!pg_exec ($query)) {
            echo "<hr>Problems initiating the database.  
                  Please try the manual procedure.</h3>";
            pg_close ($db_conn);
            exec ("$dropdb $db_name -U $db_user");
            exit ();
         }
         pg_close ($db_conn);
	 require("$localdir/includes/channel_inc.php");
	 //require ("$localdir/includes/config_inc.php");
         config_form (true);
      }
   }
   else {
      echo "<h3>Failed to make a new database. </h3> <br>";
      echo "Maybe <b>$db_user</b> is not a valid postgresql user?<br>\n";
      echo "Otherwise, you might have to temporarily disable password 
            protection of your postgresql set-up and try again.<br>";
      echo "This can be done changing the configuration in the postgres file ".
           "pg_hba.conf (tipically under /usr/local/pgsql/data) and adding two ".
           "'trust' lines like these in place of any 'ident' line:<pre>".
           "  local   all         all                                    trust".
           "\n  host    all         all      127.0.0.1   255.255.255.255   trust".
        "</pre> and then restarting the postgres daemon with 'pg_ctl restart'. ".
        "Usually this has to be done as user 'postgres'. <br>Alternatively, you can ".
        "create the database manually from the shell, forcing asking for the ".
        "postgres password by commanding <pre>".$manualcommand."</pre>".
        " Read the postgres documentation at http://www.postgresql.org/";
   }
}


if ($action=="submit") {
   //Auth::allowonly ($G_SUPER);
   printheader ();
   process_config_form ();
   printfooter ();
}

if ($action == "Vacuum") {
   Auth::allowonly ($G_SUPER);
   printheader ();
   include ("$include_dir/expire_inc.php");
   printfooter ();
}


?>
