<?php

  /**************************************************************************\
  * userdel.php     Deletes uer entry from database                          *
  ***************************************************************************
  * This script checks whether the user to be deletes has images in the      *
  *   database, and -depending on the outcome - presents different options   *
  * of how to deal with the images found.                                    *
  * This script should be integrated into adminusers.php                     *
  *                                                                          *
  * Created 17-08-2000 by Nico                                               *
  * If a user does not have images, his database entry is removed, after     *
  *   double checking                                                        *
  * When a user has only private images, these private images and the user's *
  *   dbase entry are removed                                                *
  * When a user has 'public' and private images the choice is offered to     *
  *   a. inactivate the account.  private images are deleted, and login is   *
  *      not possible anymore                                                *
  *   b. remove the account.  Private and public images are deleted          *
  *                                                                          *
  * Remainder of updates in CVS                                              *
  *                                                                          *
  * ------------------------------------------------------------------------ *
  *                                                                          *
  * Authors: SVI group University Utrecht and Nico Stuurman                  *
  * email: stuurman@rulbim.leidenuniv.nl                                     *
  * demo at: https://impi.leidenuniv.nl                                      *
  * Last modified 12/17/00 by Nico Stuurman                                  *
  * Copyright (c) 2000 by Nico Stuurman                                      *
  * ------------------------------------------------------------------------ *
  *  This program is free software; you can redistribute it and/or modify it *
  *  under the terms of the GNU General Public License as published by the   *
  *  Free Software Foundation; either version 2 of the License, or (at your  *
  *  option) any later version.                                              *
  \**************************************************************************/                                                                     


require("include.php");
Auth::allowonly($G_ADMIN);

// register variables
$get_vars = "id,";
globalize_vars ($get_vars,"HTTP_GET_VARS");
$post_vars = "id,del_with_no_images,del_with_private_images,inactive,";
globalize_vars ($post_vars, "HTTP_POST_VARS");
$PHP_SELF = $HTTP_SERVER_VARS["PHP_SELF"];

$title = "Delete a User account";

printheader();


/*********************main body*************************************/

// first check from where the script was called:
if ($del_with_no_images) {
   // user had no images, delete account?
   if ($del_with_no_images=="Yes") {
      $test=true;
      db_exec ("BEGIN WORK;");

      // delete templates
      if (! (db_exec ("DELETE FROM template WHERE userid=$id;") ))
         $test= false;; 
      // and then the account    
      if (! ($user_name = del_user_from_database($id) ) )
         $test= false;
      if ($test) {
         db_exec ("COMMIT WORK");
         echo "The account of <b>$user_name</b> has been removed<br>";
      }
      else {
         db_exec ("ROLLBACK WORK");
         echo "Internal error.  Nothing has been deleted.<br>";
      }
   }        
   else {
      // link back to 'users'
      echo "<a href=adminusers.php>Back to users</a>";
   }
}

elseif ($del_with_private_images) {
   // user had only private images, delete account?
   if ($del_with_private_images=="Yes") {
      // first remove the images
      $query = "SELECT imageid FROM images where userid=$id AND private='t';";
      $result = do_database_query ($query);
      db_exec ("BEGIN WORK");
      $test = true;
      for ($i=0; $i < db_numrows($result); $i++)
         if (! (imgdel (db_result ($result, $i, "imageid")) ))
            $test = false;
      // if there are templates these should be removed too
      db_exec ("DELETE FROM template WHERE userid=$id;");
    
      // and then the account
      if (! ($user_name = del_user_from_database($id) ) )
         $test=false; 
      if ($test) {
         db_exec ("COMMIT WORK");
         echo "The account of <b>$user_name</b> and all his/her private images have been removed";
      }
      else {
         db_exec ("ROLLBACK WORK");
         echo "Problems were encountered.  No images or user accounts have been removed.";
      }
   } 
          
   else {
      // Admin opted out, so link back to 'users'
      echo "<a href=adminusers.php>Back to users</a>";
   }
}

elseif ($inactive) {
   if ($inactive == Inactivate) {
      // last check whether this is legal
      $db_query = "SELECT login, groupid, real_name FROM users WHERE userid='$id'";
      $db_result = do_database_query($db_query);
      $db_rows = db_numrows($db_result);

      // if user exists
      if ($db_rows == 1) {

         // final check whether current user has the right to do this nasty thing
         $db_row_result = db_fetch_array ($db_result, 0);  
         if ( ($S_GROUPID==$db_row_result["groupid"]) || ($S_GROUPID==0) ) {  
            // remove private images
            $query = "SELECT imageid FROM images where userid=$id AND private='t';";
            $result = do_database_query ($query);
            db_exec ("BEGIN WORK");
            $test = true;
            for ($i=0; $i < db_numrows($result); $i++)
            if (! (imgdel (db_result ($result, $i, "imageid")) ))
               $test = false;
            // Inactivate the account

          $query2 = "UPDATE users SET active='f' WHERE userid='$id'";
          if (do_database_query($query2)) {
         
             echo "The account of <b>$db_row_result[2]</b> has been inactivated<br>";
          }
          else
             $test = false;
          }
          if ($test) db_exec ("COMMIT WORK;");
          else db_exec ("ROLLBACK WORK;"); 
      }
   }
   elseif ($inactive=="Re-activate") {
      // check wether current user has the right to do this
      $db_query = "SELECT login, groupid, real_name FROM users WHERE userid='$id'";
      $db_result = do_database_query($db_query);
      $db_rows = db_numrows($db_result);

      // if user exists
      if ($db_rows == 1) {

         // final check whether current user has the right to do this nasty thing
         $db_row_result = db_fetch_array ($db_result, 0);  
         if ( ($S_GROUPID==$db_row_result["groupid"]) || ($S_GROUPID==0) ) {
            $query2 = "UPDATE users SET active='t' WHERE userid='$id'";
            if (do_database_query($query2)) 
               echo "The account of <b>$db_row_result[2]</b> has been re-activated<br>";
         }
      }
   }
   elseif ($inactive=="Delete") {
      // last check whether this is legal
      $db_query = "SELECT login, groupid, real_name FROM users WHERE userid='$id'";
      $db_result = do_database_query($db_query);
      $db_rows = db_numrows($db_result);

      // if user exists
      if ($db_rows == 1) {

         // final check whether current user has the right to do this nasty thing
         $db_row_result = db_fetch_array ($db_result, 0);  
         if ( ($S_GROUPID==$db_row_result["groupid"]) || ($S_GROUPID==0) ) {  
            // remove private images
            $query = "SELECT imageid FROM images where userid=$id;";
            $result = do_database_query ($query);
            db_exec ("BEGIN WORK");
            $test = true;
            for ($i=0; $i < db_numrows($result); $i++)
               // delete the images
               if (! (imgdel (db_result ($result, $i, "imageid")) ))
                  $test = false;

            // if there are templates, delete them
            if (! (db_exec ("DELETE FROM template WHERE userid=$id;")))
                $test=false;

            // Delete the account
            if (! ($user_name = del_user_from_database($id) ) )
               $test=false; 
         }
         if ($test) {
            db_exec ("COMMIT WORK;");
            echo "The account of <b>$user_name</b> and all his/her images have been removed";
         }
         else {
            db_exec ("ROLLBACK WORK");
            echo "Problems were encountered.  No images or user accounts have been removed.";
         }   
      }
   }

   elseif ($inactive=="Cancel") {
      // Admin opted out, so link back to 'users'
      echo "<a href=adminusers.php>Back to users</a>";
   }
   
}
   



// script was not called from within, so check what's up with the victim
else {

   //check wether or not the victim still owns images
   $query = "SELECT imageid, filename FROM images WHERE userid = $id";
   if (!($result = do_database_query ($query) ))  
      die ("");
   $number_of_images = db_numrows($result);
   
   if ($number_of_images==0) {
      echo "This user does not have any images in the database<br>";
      echo "Are you sure the account should be deleted?<br>";
       
      echo "<FORM ENCTYPE=\"multipart/form-data\" METHOD=\"post\" ACTION=\"$PHP_SELF\">";
      // add the id of the poor soul to the form data 
      echo "<input type=\"hidden\" name=\"id\" value=\"$id\">\n";
      // and give a moment of respite
      echo "<input type=\"submit\" name=\"del_with_no_images\" value=\"Yes\">\n";
      echo "<input type=\"submit\" name=\"del_with_no_images\" value=\"No\">\n";
      echo "</FORM>\n";
   }
   
   // the victim still has images
   else {
      // check whether all images are private
      $query = "SELECT imageid FROM images where userid=$id AND private='t';";
      $result = do_database_query ($query);
      $number_of_private_images = db_numrows($result);
        
      if ($number_of_images==$number_of_private_images) {
         // all images are private:
          
         echo "This user has only private images in the database.<br>";
         echo "When you delete the account all these private images will also be deleted<br>";
         echo "Are you sure the account should be deleted?<br>";
   
         echo "<FORM ENCTYPE=\"multipart/form-data\" METHOD=\"post\" ACTION=\"$PHP_SELF\">";
         // add the id of the poor soul to the form data 
         echo "<input type=\"hidden\" name=\"id\" value=\"$id\">\n";
         // and give a moment of respite
         echo "<input type=\"submit\" name=\"del_with_private_images\" value=\"Yes\">\n";
         echo "<input type=\"submit\" name=\"del_with_private_images\" value=\"No\">\n";
         echo "</FORM>\n";
      }
      
      
      else {
         // private and public images
         echo "This user has public and possibly also private images in the database.<br><br>";

         // check whether the account is still active
         $query = "SELECT active FROM users where userid=$id;";
         $result = do_database_query ($query);
         $result_active = db_fetch_array ($result,0);
         if ($result_active["active"]=='f') $active = false;
         else $active = true;

         echo "<Table width='80%'>";
         echo "<Col class='center'><Col>";
         echo "<FORM ENCTYPE=\"multipart/form-data\" METHOD=\"post\" ACTION=\"$PHP_SELF\">";
         // add the id of the poor soul to the form data 
         echo "<input type=\"hidden\" name=\"id\" value=\"$id\">\n";
         // and give a moment of respite

         echo "<TR><TD>\n";
         if ($active) {
            echo "<input type=\"submit\" name=\"inactive\" value=\"Inactivate\">\n";
            echo "<TD>";
            echo "When you inactivate the account, all private images will be removed, ";
            echo "public images will be saved, and login into the account will be disallowed.<br>\n";
         }
         else {
            echo "<input type=\"submit\" name=\"inactive\" value=\"Re-activate\">\n";
            echo "<TD>";
            echo "Re-allows login into the account using the old password ";
         }
         echo "<TR><TD>\n";
         echo "<input type=\"submit\" name=\"inactive\" value=\"Delete\">\n";
         echo "<TD>Deleting the account will remove all images for ever. ";
         echo "Only press this button when you are absolutely certain!";

         echo "<TR><TD>\n";   
         echo "<input type=\"submit\" name=\"inactive\" value=\"Cancel\">\n";

         echo "</FORM>\n";
         echo "</Table>\n";
      }
   } 
} 
printfooter (); 
?>
